#!/usr/bin/python

import signal
import sys, os, copy

##
## I18N
## 
### gettext ("_") must come before import gtk ###
import gettext
PROGNAME="redhat-config-cluster"
gettext.bindtextdomain(PROGNAME, "/usr/share/locale")
gettext.textdomain(PROGNAME)
_=gettext.gettext

### gettext first, then import gtk (exception prints gettext "_") ###
try:
    import gtk
    import gtk.glade
except RuntimeError, e:
    print _("""
  Unable to initialize graphical environment. Most likely cause of failure
  is that the tool was not run using a graphical environment. Please either
  start your graphical user interface or set your DISPLAY variable.

  Caught exception: %s
""") % e
    sys.exit(-1)

import gnome
import gnome.ui
import gobject

from clusterpkg.cluconfig_module import cluconfig
import daemonPropDialog
import sharedstateDialog
import membersTreeview
import servicesTreeview
import failoverdomainsListview
import smbShareDruid
import nfsExportDruid
from clusterpkg.cluster_functions     import OptionError, \
                                             getStatusOutput, \
                                             initialize_shared_state
from clusterpkg.cluster_functions_gui import errorMessage, warningMessage
from clusterpkg.util_smbconf_module   import smbConf, getAbsShareFilename
from clusterpkg.util_lock_module      import lockFileException

INSTALLDIR="/usr/share/redhat-config-cluster/configure"
sys.path.append(INSTALLDIR)

VERSION = "1.0.3"
gnome.program_init(PROGNAME, VERSION)

RHCC_VERSION=_("%s %s") % ('redhat-config-cluster', VERSION)
### Get Clumanager version to present in 'About' ###
(status, output) = getStatusOutput("clustat -v")
if status == 0:
    CLUMANAGER_VERSION = _("clumanager %s") % output
else:
    CLUMANAGER_VERSION = ""
if CLUMANAGER_VERSION != "":
    ABOUT_VERSION = RHCC_VERSION + "\n" + CLUMANAGER_VERSION
else:
    ABOUT_VERSION = RHCC_VERSION

FORMALNAME=_("redhat-config-cluster")

### PC = Power Controller
PC_COLUMN_IPADDR=0
PC_COLUMN_TYPE=1
PC_COLUMN_DATA=2

gnome.program_init (PROGNAME,VERSION)

TRUE = (1==1)
FALSE = not TRUE

def initGlade():
    gladepath = "cluster-config.glade"
    if not os.path.exists(gladepath):
        gladepath = INSTALLDIR + "/cluster-config.glade"

    gtk.glade.bindtextdomain(PROGNAME)
    glade_xml = gtk.glade.XML (gladepath, domain=PROGNAME)
    return glade_xml

def runFullGUI():
    signal.signal (signal.SIGINT, signal.SIG_DFL)
    if os.getuid() != 0:
        print _("Please restart %s with root permissions!") % (sys.argv[0])
        sys.exit(10)

    glade_xml = initGlade()
    f = fullGUI(glade_xml)
    app = glade_xml.get_widget('clusterConfigApp')
    app.show_all()
    gtk.mainloop()

def sambaSharesSnapshot(clu):
    """Take a snapshot of the current Samba Shares defined in the cluster, store it in a list(just share names), and return the list."""
    services_entity = clu.getservices()
    services = services_entity.get_list()
    #share_list = []
    #for service in services:
    #    devices = service.get_devices()
    #    for device in devices:
    #        if device.get("sharename") != "":
    #            share_list.append(device.get("sharename"))
    #return share_list
    device_share_dict = {}
    for service in services:
        devices = service.get_devices()
        for device in devices:
            device_name = device.get("name")
            share_name  = device.get("sharename")
            if share_name in ("", "None", "none"):
                share_name = ""
            device_share_dict[device_name] = share_name
    return device_share_dict

class fullGUI:
    def __init__(self, glade_xml):
        self.glade_xml = glade_xml

        try:
            ### Parse the cluster.xml into DOM nodes, and lock the cluster.xml
            self.clu = cluconfig(lock=TRUE)
        except lockFileException, e:
            dlg = gtk.MessageDialog(
                    parent         = None,
                    flags          = gtk.DIALOG_MODAL,
                    type           = gtk.MESSAGE_ERROR,
                    buttons        = gtk.BUTTONS_OK,
                    message_format = e.getMessage())
            dlg.set_position(gtk.WIN_POS_MOUSE)
            dlg.show_all()
            rc = dlg.run()
            dlg.destroy()
            sys.exit(1)

        self.clu_saved = cluconfig(lock=FALSE) # Used to see if data
                                               # changed when user quits.

        ### constants for notebook pages ###
        self.MEMBERS = 0
        self.FAILOVERDOMAINS = 1
        self.SERVICES = 2

        self.glade_xml.signal_autoconnect(
            {
              "on_contents_activate" : self.on_contents_activate,
              "on_save_activate" : self.on_save_activate,
              "on_daemon_properties_activate" : self.on_daemon_properties_activate,
              "on_sharedstate_activate" : self.on_sharedstate_activate,
              "on_quit_activate" : self.quit,
              "on_about_activate" : self.on_about_activate,

              "on_notebook_switch_page" : self.on_notebook_switch_page,

              "on_add_button_clicked" : self.on_add_activate,
              "on_properties_button_clicked" : self.on_properties_activate,
              "on_delete_button_clicked" : self.on_delete_activate,
              "on_add_child_button_clicked" : self.on_add_child_activate,
              "on_bulkload_button_clicked" : self.on_bulkload_button_clicked,

              "on_add_samba_activate" : self.on_add_samba_activate,
              "on_add_nfs_activate" : self.on_add_nfs_activate,
            }
        )

        # Main window
        self.winMain = glade_xml.get_widget('clusterConfigApp')
        # delete-event allows you to click the 'X'(upper right) to close
        # the window and stop the application.  It also allows you to
        # click on the down arrow(upper left) to close and stop the app.
        self.winMain.connect("delete-event", self.quit)
        self.winMain.connect("hide", self.quit)

        self.notebook = glade_xml.get_widget('notebook')

        self.glade_xml.get_widget('add_child_button').set_sensitive(FALSE)
        self.glade_xml.get_widget('add_child_menu').set_sensitive(FALSE)
        self.glade_xml.get_widget('bulkload_menu').set_sensitive(FALSE)


        ###################################
        ### Populate Cluster attributes ###
        ###################################
        self.cluster = self.clu.getcluster() # get general cluster attributes

        cluster_name = self.cluster.get("name")
        cluster_name_widget = self.glade_xml.get_widget("cluster_name_entry")
        cluster_name_widget.set_text(cluster_name)

        ########################################
        ### Create Daemon Properties Dialog  ###
        ########################################
        self.daemonPropDialog = daemonPropDialog.daemonPropDialog(self.glade_xml, self.clu)

        #################################
        ### Populate Members Treeview ###
        #################################
        members_entity = self.clu.getmembers()
        self.membersTree = membersTreeview.membersTreeview(self.glade_xml, members_entity)

        ##################################
        ### Populate Services Treeview ###
        ##################################
        services_entity = self.clu.getservices()
        self.servicesTree = servicesTreeview.servicesTreeview(self.glade_xml, services_entity)

        ##########################################
        ### Populate Failover Domains Listview ###
        ##########################################
        failoverdomains_entity = self.clu.getfailoverdomains()
        self.failoverdomainsList = failoverdomainsListview.failoverdomainsListview(self.glade_xml, failoverdomains_entity)

        self.sharedstate_dialog = sharedstateDialog.sharedstateDialog(self.glade_xml, self.clu)

        ##########################################################
        ### Save device 'share' (samba) info, so we can compare
        ### at the end.  In on_save_activiate, if devices or
        ### samba shares were deleted, we must then delete the
        ### /etc/samba/smb.conf.sharename files.
        ##########################################################
        #self.share_list_before = sambaSharesSnapshot(self.clu)
        self.device_share_dict_before = sambaSharesSnapshot(self.clu)

        ####################################
        ### Save NFS export druid dialog ###
        ####################################
        self.nfsExportDruid = nfsExportDruid.nfsExportDruid(self.glade_xml, self.clu)

        ######################################
        ### Save samba export druid dialog ###
        ######################################
        self.smbShareDruid = smbShareDruid.smbShareDruid(self.glade_xml, self.clu)

    def process_cluster_options(self):
        name = self.glade_xml.get_widget('cluster_name_entry').get_text()
        if name == self.cluster.get("name"):
            return # none of cluster options changed - just return

        ### At least 1 cluster option changed ###
        options = { "name" : name }

        self.refresh = TRUE
        rc = gtk.RESPONSE_OK
        try:
            ### Invoke service 'validate' to validate all options
            self.cluster.validate(options)
        except OptionError, e:
            if e.getSeverity() == 'FATAL':
                errorMessage(e.getMessage())
                self.refresh = FALSE # Don't refresh - leave bad values
                                         # on screen.
                rc = gtk.RESPONSE_REJECT

            elif e.getSeverity() == 'WARNING':
                msg = _("%s\n\nPress 'OK' to Continue, or  'Cancel' to Exit.") % e.getMessage()
                rc = warningMessage(msg)
                if rc != gtk.RESPONSE_OK:
                   self.refresh = FALSE # Don't refresh - leave bad values
                                            # on screen.

        if rc != gtk.RESPONSE_OK:
            return

        ### Validation passed - now 'set' DOM XML options ###
        self.cluster.set("name", name)

    def updateSambaShareFiles(self):
        servicesTreeview = self.servicesTree
        #for sharename in self.share_list_before:
        #    if not servicesTreeview.shareExists(sharename):
        #        ### sharename existed before, but does not exist in
        #        ### current gui servicesTreeview, so the user deleted it -
        #        ### so remove the corresponding /etc/samba/smb.conf.[sharename]
        #        ### file now.
        #        try:
        #            os.remove(getAbsShareFilename(sharename))
        #        except OSError, e:
        #            pass

        #servicesTreeview.saveAllShares()
        servicesTreeview.saveAllShares(self.device_share_dict_before)

        #self.share_list_before = sambaSharesSnapshot(self.clu)
        self.device_share_dict_before = sambaSharesSnapshot(self.clu)

    def on_contents_activate(self,args):
        page = "file:///usr/share/doc/redhat-config-cluster-" + VERSION + "/ch1-software-overview.html"
        gnome.url_show(page)

    def on_save_activate(self, *args):
        #print "on_save_activate self = %s" % self
        self.process_cluster_options()
        self.clu.saveXMLToFile()

        self.clu_saved = cluconfig(lock=FALSE) # self.clu_saved used to see if
                                               # data changed when user quits.

        self.updateSambaShareFiles()

    def check_shared_storage(self):
        #dlg = self.glade_xml.get_widget('initSharedStorageDialog')
        if not self.clu.created_config:
            (status, output) = getStatusOutput("shutil -p /cluster/header")
            if status == 0:
                ### Shared Storage was previously initialized and is
                ### responding fine.
                return TRUE

        ### Shared Storage is NOT responding - maybe it hasn't been
        ### initialized yet.
        if self.clu.created_config:
            msg = "Cluster Configuration file created!\n\nPress 'OK' to initialize Shared Storage"
        else:
            msg = "Shared Storage is not responding!\n\nPress 'OK' to re-initialize Shared Storage"
        dlg = gtk.MessageDialog(parent=None,
                                flags=0,
                                type=gtk.MESSAGE_QUESTION,
                                buttons=gtk.BUTTONS_OK,
                                message_format=msg)
        dlg.set_position(gtk.WIN_POS_MOUSE)
        dlg.show_all()
        rc = dlg.run()
        dlg.hide()
        if rc == gtk.RESPONSE_OK:
            try:
                initialize_shared_state(self.clu)
            except OptionError, e:
                errorMessage(e.getMessage())
                return FALSE

        return TRUE

    def quit(self, *args):
        self.process_cluster_options() # Make sure changes to cluster option
                                       # 'name' gets recorded in the XML.
        ### Make sure at least 1 member exists, or give warning ###
        members_list = self.clu.getmembers().get_list()
        if len(members_list) == 0:
            #print "Quit: No members defined for cluster!"
            dlg = self.glade_xml.get_widget('noMembersDialog')
            dlg.show_all()
            rc = dlg.run()
            dlg.hide()
            if rc == gtk.RESPONSE_CANCEL:
                ### User doesn't want to Quit without defining at least
                ### 1 member - continue app.
                return

        ### See if any data changed ###
        if self.clu == self.clu_saved:
            ### No data changed ###
            if not self.check_shared_storage():
                ### Problem initializing Shared Storage - back to app ###
                return TRUE
            gtk.mainquit()
            return

        ### Data has changed ###
        dlg = self.glade_xml.get_widget('quitSaveChangesDialog')
        dlg.show_all()
        rc = dlg.run()
        dlg.hide()
        if rc == gtk.RESPONSE_NO:
            ### User pressed 'No' (Exit without saving)
            if self.clu.created_config:
                ### Config file was created, but user pressed 'No',
                ### so remove it.
                try:
                    os.remove(self.clu.file)
                except OSError, e:
                    pass
            gtk.mainquit()
            return
        elif rc == gtk.RESPONSE_CANCEL:
            ### User pressed 'Cancel' (return to Cluster Config)
            pass
        else:
            ### User pressed 'Yes' (Save and Exit)
            self.on_save_activate()
            if not self.check_shared_storage():
                ### Problem initializing Shared Storage - back to app ###
                return TRUE
            gtk.mainquit()
            return

    def on_about_activate(self, *args):
        dialog = gnome.ui.About(
            ABOUT_VERSION,
            '', ### Don't specify version - already in ABOUT_VERSION
            _("Copyright (c) 2003 Red Hat, Inc. All rights reserved."),
            _("This software is licensed under the terms of the GPL."),
            [ 'Hardy Merrill (redhat-config-cluster) <hmerrill at redhat.com>',
              'Daniel Walsh (redhat-config-cluster) <dwalsh at redhat.com>',
              '',
              'Brian Stevens (clumanager) <bstevens at redhat.com>',
              'Tim Burke (clumanager) <tburke at redhat.com>',
              'Lon Hohberger (clumanager) <lhh at redhat.com>',
              '',
              'Portions of Red Hat Cluster Manager (clumanager) based on Kimberlite v1.1.0 (C) 2000-2001 Mission Critical Linux, Inc.' ],     
            [ 'Valerie Rodgers <vrodgers at redhat.com>',
              'John Ha <jha at redhat.com>',
              'Tammy Fox <tfox at redhat.com>' ], # doc people
        ) ### end dialog
        dialog.set_title (FORMALNAME)
        dialog.show()

    def on_daemon_properties_activate(self, *args):
        #print "\nMain: on_daemon_properties_activate TOP now!"
        #print "============================================"
        #print "  XML = %s" % self.clu.dom.toxml()
        while 1:
            rc = self.daemonPropDialog.run()
            if rc == gtk.RESPONSE_OK:
                ### Either
                ###   1. User clicked daemonPropDialog 'OK' and
                ###      validation passed and data was written,
                ###        OR
                ###   2. User clicked daemonPropDialog 'OK' and
                ###      validation *FAILED* with a WARNING, and on
                ###      that WARNING dialog the user clicked 'OK'
                ###      to acknowledge warning but write data anyway.
                break

            elif rc == gtk.RESPONSE_CANCEL:
                ### User 'Cancel'led out of daemonPropDialog.
                break

            elif rc == gtk.RESPONSE_REJECT:
                ### 1. User got a FATAL error and clicked 'OK' (only option)
                ###    so display same dialog again,
                ###      OR
                ### 2. User got a WARNING and clicked 'Cancel' to go back
                ###    and re-enter.
                continue

        self.daemonPropDialog.hide ()

    def on_sharedstate_activate(self, *args):
        #print "\nMain: on_sharedstate_activate TOP now!"
        #print "======================================"
        #print "  XML = %s" % self.clu.dom.toxml()
        while 1:
            rc = self.sharedstate_dialog.run()
            if rc == gtk.RESPONSE_OK:
                break
            elif rc == gtk.RESPONSE_CANCEL:
                break
            elif rc == gtk.RESPONSE_REJECT:
                continue

        self.sharedstate_dialog.hide()

        #print "\nMain: on_sharedstate_activate BOTTOM now!"
        #print "========================================="
        #print "  XML = %s" % self.clu.dom.toxml()

    #
    # args == [ Notebook, Current Page, Page # that we are changing *to* ]
    #
    def on_notebook_switch_page(self,
                                my_notebook,
                                current_page,
                                page_changing_to):
        self.glade_xml.get_widget('add_child_button').set_sensitive(FALSE)
        self.glade_xml.get_widget('add_child_menu').set_sensitive(FALSE)
        self.glade_xml.get_widget('bulkload_menu').set_sensitive(FALSE)
        if page_changing_to == self.SERVICES:
            services_list = self.servicesTree.services_entity.get_list()
            if len(services_list) > 0:
                self.glade_xml.get_widget('add_child_button').set_sensitive(TRUE)
                self.glade_xml.get_widget('add_child_menu').set_sensitive(TRUE)

    def on_add_activate(self, *args):
        #print "\nMain: on_add_activate TOP now!"
        #print "=============================="
        #print "  XML = %s" % self.clu.dom.toxml()
        current_page=self.notebook.get_current_page()
        if current_page == self.MEMBERS:
            self.membersTree.add()
        elif current_page == self.SERVICES:
            self.servicesTree.add()
        else:
            self.failoverdomainsList.add()
        #print "\nMain: on_add_activate BOTTOM now!"
        #print "=================================="
        #print "  XML = %s" % self.clu.dom.toxml()

    def on_bulkload_button_clicked(self, *args):
        return self.servicesTree.on_bulkloadNFS()
        
    def on_add_child_activate(self, *args):
        ### The "Add Child" button is only active
        ### on the "Service" notebook tab - don't
        ### need to test the other notebook tabs.
        #print "\nMain: on_add_child_activate TOP now!"
        #print "===================================="
        #print "  XML = %s" % self.clu.dom.toxml()
        current_page=self.notebook.get_current_page()
        if current_page == self.MEMBERS:
            self.membersTree.add_child()
        else:
            self.servicesTree.add_child()
        #print "\nMain: on_add_child_activate BOTTOM now!"
        #print "======================================="
        #print "  XML = %s" % self.clu.dom.toxml()

    def on_properties_activate(self, *args):
        #print "\nMain: on_properties_activate TOP now!"
        #print "====================================="
        #print "  XML = %s" % self.clu.dom.toxml()
        current_page=self.notebook.get_current_page()
        if current_page == self.MEMBERS:
            self.membersTree.properties()
        elif current_page == self.SERVICES:
            self.servicesTree.properties()
        else:
            self.failoverdomainsList.properties()
        #print "\nMain: on_properties_activate BOTTOM now!"
        #print "========================================"
        #print "  XML = %s" % self.clu.dom.toxml()

    def on_delete_activate(self, *args):
        #print "\nMain: on_delete_activate TOP now!"
        #print "================================="
        #print "  XML = %s" % self.clu.dom.toxml()
        current_page=self.notebook.get_current_page()
        if current_page == self.MEMBERS:
            self.membersTree.delete()
        elif current_page == self.SERVICES:
            self.servicesTree.delete()
            services_left_list = self.servicesTree.services_entity.get_list()
            if len(services_left_list) == 0:
                self.glade_xml.get_widget('add_child_button').set_sensitive(FALSE)
                self.glade_xml.get_widget('add_child_menu').set_sensitive(FALSE)
        else:
            self.failoverdomainsList.delete()
        #print "\nMain: on_delete_activate BOTTOM now!"
        #print "===================================="
        #print "  XML = %s" % self.clu.dom.toxml()


    def on_add_nfs_activate(self, *args):
        #print "\nMain: on_add_nfs_activate TOP now!"
        #print "=================================="
        #print "  XML = %s" % self.clu.dom.toxml()
        rc = self.nfsExportDruid.run()
        #print "\nMain: on_add_nfs_activate BOTTOM now!"
        #print "====================================="
        #print "  XML = %s" % self.clu.dom.toxml()

        if rc == gtk.RESPONSE_OK:
            ### User added an NFS Export - need to rebuild Services Treeview
            ### so that GUI incorporates new export.
            self.servicesTree.populate_treeview()

    def on_add_samba_activate(self, *args):
        #print "\nMain: on_add_samba_activate TOP now!"
        #print "===================================="
        #print "  XML = %s" % self.clu.dom.toxml()
        rc = self.smbShareDruid.run()
        #print "\nMain: on_add_samba_activate BOTTOM now!"
        #print "======================================="
        #print "  XML = %s" % self.clu.dom.toxml()

        if rc == gtk.RESPONSE_OK:
            ### User added an Samba Share - need to rebuild Services Treeview
            ### so that GUI incorporates new stuff.
            self.servicesTree.populate_treeview()

if __name__ == "__main__":
    runFullGUI()
