#!/usr/bin/python
from clusterpkg.cluster_module    import cluster
from clusterpkg.sharedstate_module import sharedstate
from clusterpkg.clumembd_module   import clumembd
#from clusterpkg.clupowerd_module  import clupowerd
from clusterpkg.cluquorumd_module import cluquorumd
from clusterpkg.clurmtabd_module  import clurmtabd
from clusterpkg.clusvcmgrd_module import clusvcmgrd
from clusterpkg.clulockd_module   import clulockd
from clusterpkg.members_module    import members
from clusterpkg.services_module   import services
from clusterpkg.failoverdomains_module import failoverdomains
from clusterpkg.cluster_functions import getStatusOutput
from clusterpkg.util_lock_module  import lockFile, lockFileException

import sys
import shutil # for copyfile
import os     # for rename

from xml.dom import minidom

from clusterpkg.ClusterList import ClusterList

import clusterpkg.sharedstate_module
import clusterpkg.clumembd_module
#import clusterpkg.clupowerd_module
import clusterpkg.cluquorumd_module
import clusterpkg.clurmtabd_module
import clusterpkg.clusvcmgrd_module
import clusterpkg.clulockd_module
import clusterpkg.cluster_module
import clusterpkg.members_module
import clusterpkg.services_module
import clusterpkg.failoverdomains_module
SHAREDSTATE_INFO = clusterpkg.sharedstate_module.MY_INFO
CLUHBD_INFO      = clusterpkg.clumembd_module.MY_INFO
#CLUPOWERD_INFO   = clusterpkg.clupowerd_module.MY_INFO
CLUQUORUMD_INFO  = clusterpkg.cluquorumd_module.MY_INFO
CLURMTABD_INFO   = clusterpkg.clurmtabd_module.MY_INFO
CLUSVCMGRD_INFO  = clusterpkg.clusvcmgrd_module.MY_INFO
CLULOCKD_INFO    = clusterpkg.clulockd_module.MY_INFO
CLUSTER_OPTIONS_INFO = clusterpkg.cluster_module.MY_INFO
MEMBERS_INFO     = clusterpkg.members_module.MY_INFO
SERVICES_INFO    = clusterpkg.services_module.MY_INFO
FAILOVERDOMAINS_INFO = clusterpkg.failoverdomains_module.MY_INFO

TRUE = (1==1)
FALSE = not TRUE
MY_INFO = {
   "WHOAMI" : "cluconfig",
   "OPTIONS" : { },

   "ADD_GETOPTS"     : [],
   "DELETE_GETOPTS"  : [],
   "UPDATE_GETOPTS"  : [],
   "LISTALL_GETOPTS" : [ "cluconfig" ],
}

##########################################################################
# cluconfig: This class inherits from ClusterList so when 'cluconfig'
#            or 'redhat-config-cluster' runs for the 1st time
#            (/etc/cluster.xml doesn't exist), class 'cluconfig'
#            can call ClusterList's 'append_child' method to create
#            nodes clumembd, clupowerd, cluquorumd, clurmtabd,
#            clusvcmgrd, clulockd, cluster, which make up the minimum
#            skeleton of /etc/cluster.xml.
##########################################################################
#
class cluconfig(ClusterList):
   def __cmp__(self, other):
      ### Called when 2 cluconfig objects are compared ###
      integer_rc = self.dom.toxml().__cmp__(other.dom.toxml())
      #print "cluconfig __cmp__: self  =", self
      #print "cluconfig __cmp__: other =", other
      #print "cluconfig __cmp__: rc =", integer_rc
      return integer_rc

   def __init__(self, node=None, lock=TRUE):
      self.lock = lock
      if self.lock:
         try:
            self.lockfile = lockFile("Red Hat Cluster Configuration", "cluster.xml")
         except lockFileException:
            self.lock = FALSE # in destructor __del__, don't release since
                              # didn't get lock.
            raise

      self.file = "/etc/cluster.xml"
      try:
         self.dom = minidom.parse(self.file)

         ### If parse passes(cluster.xml exists), then call
         ### ClusterList constructor.  In this situation, we
         ### won't really be using cluconfig as a ClusterList
         ### object - it will be used below when the cluster.xml
         ### file doesn't exist, since ClusterList objects can
         ### call "add_child".
         my_node = None
         parent = None
         ClusterList.__init__(self, MY_INFO, my_node, parent)
         self.created_config = FALSE
      except IOError, e:
         ### /etc/cluster.xml file does *NOT* exist - create one ###
         new_doc = minidom.Document()

         ### now create the 'root' element - named 'cluconfig' ###
         new_element = new_doc.createElement("cluconfig")
         root_node = new_doc.appendChild(new_element)
         root_node.setAttribute("version", "3.0")

         parent = None
         ClusterList.__init__(self, MY_INFO, root_node, parent)

         self.add_clumembd()
         #self.add_clupowerd()
         self.add_cluquorumd()
         self.add_clurmtabd()
         self.add_clusvcmgrd()
         self.add_clulockd()
         self.add_cluster_options()
         self.add_sharedstate()
         self.add_members()
         self.add_services()
         self.add_failoverdomains()
         
         f = open(self.file, "w")
         f.write(new_doc.toxml())
         f.close()

         ### Now that /etc/cluster.xml has been created, generate
         ### an authentication key - tag 'cluster' option 'key'.
         (status, output) = getStatusOutput("clukey generate")

         self.dom = minidom.parse(self.file)

         self.created_config = TRUE

   def __del__(self):
      ### this is the destructor ###
      #print "cluconfig __del__ TOP!"
      if self.lock:
         #print "  release lock now!"
         self.lockfile.releaseLock()

   def add_sharedstate(self, cmdline_args=None):
      ### First, create a new 'sharedstate' entity
      entity = ClusterList.add_child(self, SHAREDSTATE_INFO)
      sharedstate_entity = sharedstate(entity.node)
      sharedstate_entity.add_options(cmdline_args)
      ### Now, remove the "id" attribute that "add_child" created.
      ### We don't want this since there's only 1 sharedstate node - "id"s
      ### exist on nodes that can have siblings.
      sharedstate_entity.node.removeAttribute("id")
      return sharedstate_entity

   def add_clumembd(self, cmdline_args=None):
      ### First, create a new 'clumembd' entity
      entity = ClusterList.add_child(self, CLUHBD_INFO)
      clumembd_entity = clumembd(entity.node)
      clumembd_entity.add_options(cmdline_args)
      ### Now, remove the "id" attribute that "add_child" created.
      ### We don't want this since there's only 1 clumembd node - "id"s
      ### exist on nodes that can have siblings.
      clumembd_entity.node.removeAttribute("id")
      return clumembd_entity

   #def add_clupowerd(self, cmdline_args=None):
   #   entity = ClusterList.add_child(self, CLUPOWERD_INFO)
   #   clupowerd_entity = clupowerd(entity.node)
   #   clupowerd_entity.add_options(cmdline_args)
   #   clupowerd_entity.node.removeAttribute("id")
   #   return clupowerd_entity

   def add_cluquorumd(self, cmdline_args=None):
      entity = ClusterList.add_child(self, CLUQUORUMD_INFO)
      cluquorumd_entity = cluquorumd(entity.node)
      cluquorumd_entity.add_options(cmdline_args)
      cluquorumd_entity.node.removeAttribute("id")
      return cluquorumd_entity

   def add_clurmtabd(self, cmdline_args=None):
      entity = ClusterList.add_child(self, CLURMTABD_INFO)
      clurmtabd_entity = clurmtabd(entity.node)
      clurmtabd_entity.add_options(cmdline_args)
      clurmtabd_entity.node.removeAttribute("id")
      return clurmtabd_entity

   def add_clusvcmgrd(self, cmdline_args=None):
      entity = ClusterList.add_child(self, CLUSVCMGRD_INFO)
      clusvcmgrd_entity = clusvcmgrd(entity.node)
      clusvcmgrd_entity.add_options(cmdline_args)
      clusvcmgrd_entity.node.removeAttribute("id")
      return clusvcmgrd_entity

   def add_clulockd(self, cmdline_args=None):
      entity = ClusterList.add_child(self, CLULOCKD_INFO)
      clulockd_entity = clulockd(entity.node)
      clulockd_entity.add_options(cmdline_args)
      clulockd_entity.node.removeAttribute("id")
      return clulockd_entity

   def add_cluster_options(self, cmdline_args=None):
      entity = ClusterList.add_child(self, CLUSTER_OPTIONS_INFO)
      cluster_options_entity = cluster(entity.node)
      cluster_options_entity.add_options(cmdline_args)
      cluster_options_entity.node.removeAttribute("id")
      cluster_options_entity.updateViewnumber()
      return cluster_options_entity

   def add_members(self, cmdline_args=None):
      entity = ClusterList.add_child(self, MEMBERS_INFO)
      members_entity = members(entity.node)
      members_entity.node.removeAttribute("id")
      return members_entity

   def add_services(self, cmdline_args=None):
      entity = ClusterList.add_child(self, SERVICES_INFO)
      services_entity = services(entity.node)
      services_entity.node.removeAttribute("id")
      return services_entity

   def add_failoverdomains(self, cmdline_args=None):
      entity = ClusterList.add_child(self, FAILOVERDOMAINS_INFO)
      failoverdomains_entity = failoverdomains(entity.node)
      failoverdomains_entity.node.removeAttribute("id")
      return failoverdomains_entity

#########################################################################

   def getcluster    (self):
      return cluster(self.dom.getElementsByTagName("cluster")[0])
   def getsharedstate(self):
      return sharedstate(self.dom.getElementsByTagName("sharedstate")[0])
   def getclumembd   (self):
      return clumembd(self.dom.getElementsByTagName("clumembd")[0])
   #def getclupowerd  (self):
   #   return clupowerd(self.dom.getElementsByTagName("clupowerd")[0])
   def getcluquorumd (self):
      return cluquorumd(self.dom.getElementsByTagName("cluquorumd")[0])
   def getclurmtabd  (self):
      return clurmtabd(self.dom.getElementsByTagName("clurmtabd")[0])
   def getclusvcmgrd (self):
      return clusvcmgrd(self.dom.getElementsByTagName("clusvcmgrd")[0])
   def getclulockd (self):
      return clulockd(self.dom.getElementsByTagName("clulockd")[0])
   def getmembers    (self):
      return members(self.dom.getElementsByTagName("members")[0])
   def getservices   (self):
      return services(self.dom.getElementsByTagName("services")[0])
   def getfailoverdomains (self):
      return failoverdomains(self.dom.getElementsByTagName("failoverdomains")[0])
   
   def saveXMLToFile(self):
      clu = self.getcluster()
      clu.updateViewnumber()

      ### 1. Create .tmp file with updated info.
      temp_file = self.file + '.tmp'
      f = open(temp_file, "w")
      f.write(self.dom.toxml())
      f.close()

      ### 2. Now that .tmp has been created successfully, backup
      ###    the current config file before we overwrite it.
      backup_file = self.file + '.bak'
      shutil.copyfile(self.file, backup_file)

      ### 3. Move the temp file containing the updated info into place.
      os.rename(temp_file, self.file)

      ### 4. If cluster is running, then 'Reload' so cluster re-reads
      ###    its updated config file.
      (status, output) = getStatusOutput("service clumanager status")
      if status == 0:
         ### Cluster *IS* running ###
         print "Cluster *IS* running on this node - now running 'service clumanager reload'"
         (status, output) = getStatusOutput("service clumanager reload")
      else:
         #print "   cluster is *NOT* running"
         pass

   def out(self):
      #print "cluconfig"
      self.getcluster().out()
      self.getsharedstate().out()
      self.getclumembd().out()
      #self.getclupowerd().out()
      self.getcluquorumd().out()
      self.getclurmtabd().out()
      self.getclusvcmgrd().out()
      self.getclulockd().out()
      self.getmembers().out()
      self.getservices().out()
      self.getfailoverdomains().out()
