#!/usr/bin/python
from xml.dom import minidom
from clusterpkg.cluster_functions import OptionError, cluPing
from ClusterEntity import *
import os, stat

TRUE = (1==1)
FALSE = not TRUE

MY_INFO = {
   "WHOAMI"  : "failoverdomainnode",

   "OPTIONS" : {
      "id"   : { "datatype" : "int",
                 "order"    : 0 },

      "name" : { "datatype" : "str",
                 "order"    : 1 },
   },

   "ADD_GETOPTS"     : [ "failoverdomain=",
                            "add_failoverdomainnode",
                               "name=" ],

   "DELETE_GETOPTS"  : [ "failoverdomain=",
                            "del_failoverdomainnode",
                            "failoverdomainnode=" ],

   "LISTALL_GETOPTS" : [ "failoverdomain=",
                            "failoverdomainnodes" ],

   "UPDATE_GETOPTS"  : [ "failoverdomain=",
                            "failoverdomainnode=",
                               "name=" ],
}

############################################################################

class failoverdomainnode(ClusterEntity):
   def __init__(self, node, parent=None, action="modify"):
      ClusterEntity.__init__(self, MY_INFO, node, parent, action)

   def __dup_found(self, key, value):
      prev_value = self.get(key)
      if self.action == "add" or \
         (self.action == "modify" and value != prev_value):
          failoverdomain_entity = self.parent
          search_options = {key : value}
          failoverdomainnode_being_validated = self
          if failoverdomain_entity.failoverdomainnode_found(search_options, failoverdomainnode_being_validated):
              return TRUE
      return FALSE

   ####################################################################
   # NOTE: In 'validate', WARNING errors must come *Last*, after all
   #       FATAL errors.  This is because the command line tool will
   #       override WARNING's in favor of doing the add or update,
   #       whereas the GUI will present the WARNING in a popup dialog.
   #       Since WARNINGs and ERROR's are 'raise'd, editing stops at
   #       that point.  So for the command line, we want to be sure
   #       that all FATAL checks have already been done, so that if a
   #       WARNING is raised, it can be safely overridden sinced we
   #       know that only WARNING checks remain.
   ####################################################################
   def validate(self, options):

      #######################################################################
      ########################   1. FATAL ERRORS   ##########################
      #######################################################################

      ### Check for *required* fields on "add" ###
      ### Validate "name"
      if self.action == "add":
         if not options.has_key("name"):
            raise OptionError("FATAL",
                              _("Failover Domain Node Name is required"),
                              "name",
                              None)

      ### 'genericOptionChecks' will raise an error if either the data
      ### doesn't match the datatype(str or int), or the option is
      ### not in the list of valid options for this class.
      ClusterEntity.genericOptionChecks(self, options)

      ### Validate submitted options ###
      ### Validate "name"
      if options.has_key("name"):
         name = options["name"]
         if self.__dup_found("name", name):
            raise OptionError("FATAL",
                              _("Failover Domain Node Name already exists"),
                              "name",
                              name)

         ### Verify that Domain Node Name specified exists as a current
         ### Member name in the cluster.
         ownerDocument = self.node.ownerDocument
         members_node = ownerDocument.getElementsByTagName("members")[0]
         from clusterpkg.members_module import members
         members_entity = members(members_node)
         search_options = { "name" : name }
         if not members_entity.member_found(search_options):
            raise OptionError("FATAL",
                              _("Member not found for Failover Domain Node Name '%s'") % name,
                              "name",
                              name)


      #######################################################################
      ##########################   2. WARNINGS   ############################
      #######################################################################

      # No warnings
