#!/usr/bin/python
from xml.dom import minidom
from clusterpkg.failoverdomain_module import failoverdomain
import clusterpkg.failoverdomain_module
from ClusterList import *

FAILOVERDOMAIN_INFO = clusterpkg.failoverdomain_module.MY_INFO

MY_INFO = {
   "WHOAMI"          : "failoverdomains",
      
   "OPTIONS"         : { },
     
   "ADD_GETOPTS"     : [],
   "DELETE_GETOPTS"  : [],
   "UPDATE_GETOPTS"  : [],
   "LISTALL_GETOPTS" : [ "failoverdomains" ],
}

class failoverdomains(ClusterList):
   def __init__(self, node):
      parent = None
      ClusterList.__init__(self, MY_INFO, node, parent)

   ###################################################################
   # get_list: The list returned by ClusterList.get_list contains
   # ClusterEntity's - It's not sufficient for each to be a
   # ClusterEntity since ClusterEntity's don't have access to the
   # "failoverdomain" functions.  Each failoverdomain in the list returned by
   # this "get_list" must be a *failoverdomain* entity in order to have
   # access to the "failoverdomain" functions.
   ###################################################################
   def get_list(self):
      failoverdomainList = []
      for failoverdomain_entity in ClusterList.get_list(self, FAILOVERDOMAIN_INFO):
         failoverdomainList.append(failoverdomain(failoverdomain_entity.node, self))
      return failoverdomainList

   def get_failoverdomain(self, val, key="name"):
      search_options = {key : val}
      failoverdomain_entity = ClusterList.get_child(self, FAILOVERDOMAIN_INFO, search_options)
      return failoverdomain(failoverdomain_entity.node, self)

   def failoverdomain_found(self, search_options, failoverdomain_being_validated=None):
      ### "failoverdomain_being_validated" is either the failoverdomain
      ### being modified, or the new failoverdomain just added.  We want
      ### to be sure his key info doesn't duplicate an existing failoverdomain.
      return ClusterList.child_found(self, FAILOVERDOMAIN_INFO, search_options, failoverdomain_being_validated)

   def add_failoverdomain(self, cmdline_args=None):
      entity = ClusterList.add_child(self, FAILOVERDOMAIN_INFO)
      failoverdomain_entity = failoverdomain(entity.node, self, "add")
      failoverdomain_entity.add_options(cmdline_args)
      return failoverdomain_entity

   def failoverdomain_found_on_service(self, name):
      ownerDocument = self.node.ownerDocument
      services_node = ownerDocument.getElementsByTagName("services")[0]
      from clusterpkg.services_module import services
      services_entity = services(services_node)
      for service in services_entity.get_list():
         if service.get("failoverdomain") == name:
            return TRUE
      return FALSE

   def delete_failoverdomain(self, name, cmdline_args=None):
      if self.failoverdomain_found_on_service(name):
         raise OptionError("FATAL",
                           _("Failover Domain delete not allowed - Domain Name '%s' specified by at least one service") % name,
                           "name",
                           name)
      search_options = {"name": name}
      ClusterList.delete_child(self, FAILOVERDOMAIN_INFO, search_options, cmdline_args)

   def delete_failoverdomain_by_id(self, id):
      failoverdomain = self.get_failoverdomain(id, "id")
      if self.failoverdomain_found_on_service(failoverdomain.get("name")):
         name = failoverdomain.get("name")
         raise OptionError("FATAL",
                           _("Domain delete not allowed - Domain Name '%s' specified by at least one service") % name,
                           "name",
                           name)
      ClusterList.delete_by_id(self, FAILOVERDOMAIN_INFO, id)

   def out(self):
      nodeList = self.node.getElementsByTagName(FAILOVERDOMAIN_INFO['WHOAMI'])
      for n in nodeList:
         failoverdomain(n, self).out()
