#!/usr/bin/python
from xml.dom import minidom
from clusterpkg.cluster_functions import OptionError, cluPing
from ClusterEntity import *
import os, stat, string

TRUE = 1
FALSE = 0

ALLOW_NETWORK_SERVERTECH = FALSE

VALID_SERIAL_TYPES  = ( "rps10", )

VALID_SERIAL_PORTS  = ( "0","1","2","3","4","5","6","7","8","9","All" )

### 6/17/2003: Lon asked to have 'servertech' taken out of
###            supported network powerswitch types.
VALID_NETWORK_TYPES = [ "baytech",
                        "apcmaster",
                        "wti_nps" ]
if ALLOW_NETWORK_SERVERTECH:
   VALID_NETWORK_TYPES.append("servertech")

###################################################################
# 4/15/2003 HM - "port" is valid for both 'Serial' and 'Network'
#                powercontrollers.  It is required for 'Serial'
###################################################################
MY_INFO = {
   "WHOAMI"  : "powercontroller",

   "OPTIONS" : {
      "id"        : { "datatype" : "int",
                      "order"    : 0 },

      "type"      : { "datatype" : "str",
                      "order"    : 1,
                      "default"  : "None" },

      ### "device" is valid for "serial" type's
      "device"    : { "datatype" : "str",
                      "order"    : 2 },

      ### "ipaddress" and "port" are valid for "network" type's
      "ipaddress" : { "datatype" : "str",
                      "order"    : 3 },

      "port"      : { "datatype" : "str",
                      "order"    : 4 },

      "user"      : { "datatype" : "str",
                      "order"    : 5 },

      "password"  : { "datatype" : "str",
                      "order"    : 6 },
   },

   "ADD_GETOPTS"     : [ "member=",
                            "add_powercontroller",
                               "type=",
                               "device=",
                               "ipaddress=",
                               "port=",
                               "user=",
                               "password=" ],

   "DELETE_GETOPTS"  : [ "member=",
                            "del_powercontroller",
                            "powercontroller=" ],

   "LISTALL_GETOPTS" : [ "member=",
                            "powercontrollers" ],

   "UPDATE_GETOPTS"  : [ "member=",
                            "powercontroller=",
                               "type=",
                               "device=",
                               "ipaddress=",
                               "port=",
                               "user=",
                               "password=" ],
}

############################################################################

class powercontroller(ClusterEntity):
   def __init__(self, node, parent=None, action="modify"):
      ClusterEntity.__init__(self, MY_INFO, node, parent, action)

   def get_general_type(self, type):
      if type in VALID_SERIAL_TYPES:
         return 'serial'
      elif type in VALID_NETWORK_TYPES:
         return 'network'
      elif type == "gulm-bridge":
         return 'gulm'
      else:
         return 'error'

   ####################################################################
   # NOTE: in __validate_serial and __validate_network, 
   #
   #          WARNING errors must come *Last*, after all FATAL errors.
   #
   #       This is because the command line tool will override WARNING's
   #       in favor of doing the add or update, whereas the GUI will
   #       present the WARNING in a popup dialog.  Since WARNINGs and
   #       ERROR's are 'raise'd, editing stops at that point.  So for
   #       the command line, we want to be sure that all FATAL checks
   #       have already been done, so that if a WARNING is raised, it
   #       can be safely overridden sinced we know that only WARNING
   #       checks remain.
   ####################################################################
   def __validate_serial(self, options):
   
      #######################################################################
      ########################   1. FATAL ERRORS   ##########################
      #######################################################################
   
      ### if any 'Network' options were submitted, then error ###

      ### From GUI, network and serial options will all be submitted, but
      ### network options should have null("") value.
      if options.has_key("ipaddress") and options["ipaddress"] != "":
         raise OptionError("FATAL",
                           _("IP Address is an invalid option for a Serial powercontroller"),
                           "ipaddress",
                           options["ipaddress"])
   
      ### Check for *required* fields on "add" ###
      if self.action == "add":
         if not options.has_key("device") or options["device"] == "":
            raise OptionError("FATAL",
                              _("Device is required for a Serial powercontroller"),
                              "device",
                              None)
         device = options["device"]

         ### "device" *WAS* submitted - when "device" submitted, "port"
         ### is required.
         if not options.has_key("port") or options["port"] == "":
            raise OptionError("FATAL",
                              _("Port is required for a Serial powercontroller"),
                              "port",
                              None)
         port = options["port"]

      else:
         ### This is a "modify".
         ###
         ### Check for *required* fields, that are either existing, or that
         ### are being changed to new values, on "modify"
         prev_device = self.get("device")
         if options.has_key("device"):
            device = options["device"]
         else:
            device = prev_device
         if prev_device == "" and device == "":
            raise OptionError("FATAL",
                              _("Device is required for a Serial powercontroller"),
                              "device",
                              None)

         prev_port = self.get("port")
         if options.has_key("port"):
            port = options["port"]
         else:
            port = prev_port
         if prev_port == "" and port == "":
            raise OptionError("FATAL",
                              _("Port is required for a Serial powercontroller"),
                              "port",
                              None)

      ### Validate 'Serial' options submitted ###
      if not options.has_key("device"):
         return
   
      ### Validate Device ###
      try:
         mode = os.stat(device)[stat.ST_MODE]
      except OSError, e:
         msg = _("Serial Device file \"%s\" does not exist") % device
         raise OptionError("FATAL", msg, "device", device)
   
      if not stat.S_ISCHR(mode):
         msg = _("Serial Device file must be a character special device file")
         raise OptionError("FATAL", msg, "device", device)

      if device[0:9] != '/dev/ttyS':
         msg = _("Serial Device must begin with /dev/ttyS")
         raise OptionError("FATAL", msg, "device", device)


      ### Validate Port ###
      if not port in VALID_SERIAL_PORTS:
         msg = "Serial powercontroller Port must be one of these: "
         msg += string.join(VALID_SERIAL_PORTS, ", ")
         raise OptionError("FATAL",
                           _(msg),
                           "port",
                           port)


      ### Check for Serial 'device' duplicates within the parent member ###
      prev_device = self.get("device")
      if self.action == "add" or \
         (self.action == "modify" and device != prev_device):
          dup_found = TRUE
          ### get_serial_powercontroller will raise exception if not found
          try:
             self.parent.get_serial_powercontroller(device)
          except OptionError, e:
             dup_found = FALSE
   
          if dup_found:
             raise OptionError("FATAL",
                               _("Serial powercontroller Device already exists in member"),
                               "device",
                               device)

      ### Can only allow 'Serial' powercontroller if there are 2 members
      ### in the cluster - if >2 members, then 'Serial' invalid - must
      ### use 'Network' powercontroller.
      member_entity = self.parent
      members_entity = member_entity.parent
      members_list = members_entity.get_list()
      count_members = len(members_list)
      if count_members > 2:
         raise OptionError("FATAL",
                           _("Serial powercontroller may not be used when cluster contains more than 2 members"),
                           "ipaddress",
                           None)

      #######################################################################
      ##########################   2. WARNINGS   ############################
      #######################################################################
   
      # No WARNINGS for ___validate_serial


   def __validate_network(self, options):
   
      #######################################################################
      ########################   1. FATAL ERRORS   ##########################
      #######################################################################
   
      ### if any 'Serial' options were submitted, then error ###

      ### From GUI, network and serial options will all be submitted, but
      ### serial options should have null("") value.
      if options.has_key("device") and options["device"] != "":
         raise OptionError("FATAL",
                           _("Device is an invalid option for a Network powercontroller"),
                           "device",
                           options["device"])
   
      ### Check for *required* fields on "add" ###
      if self.action == "add":
         if not options.has_key("ipaddress") or options["ipaddress"] == "":
            raise OptionError("FATAL",
                              _("IP Address is required for a Network powercontroller"),
                              "ipaddress",
                              None)
         if not options.has_key("port") or options["port"] == "":
            raise OptionError("FATAL",
                              _("Port is required for a Network powercontroller"),
                              "port",
                              None)
         ipaddress = options["ipaddress"]
         port      = options["port"]
         prev_ipaddress = None
         prev_port      = None
      else:
         ### Must be a "modify"

         ### Check for *required* fields, that are either existing, or that
         ### are being changed to new values, on "modify"
         prev_ipaddress = self.get("ipaddress")
         prev_port      = self.get("port")
         if options.has_key("ipaddress"):
            ipaddress = options["ipaddress"]
         else:
            ipaddress = prev_ipaddress

         if options.has_key("port"):
            port = options["port"]
         else:
            port = prev_port

         if prev_ipaddress == "" and ipaddress == "":
            raise OptionError("FATAL",
                              _("IP Address is required for a Network powercontroller"),
                              "ipaddress",
                              None)
         if prev_port == "" and port == "":
            raise OptionError("FATAL",
                              _("Port is required for a Network powercontroller"),
                              "port",
                              None)
   
      ### Check for Network 'ipaddress/port' duplicates within the
      ### parent member.
      if self.action == "add" or \
         (self.action == "modify" and (ipaddress != prev_ipaddress or \
                                       port      != prev_port)):
   
          ### get_network_powercontroller will raise exception if not found
          search_options = { "ipaddress" : ipaddress,
                             "port"      : port }
          member = self.parent
          dup_found = TRUE
          try:
             member.get_network_powercontroller(search_options)
          except OptionError, e:
             dup_found = FALSE
   
          if dup_found:
             raise OptionError("FATAL",
                               _("Network powercontroller IP Address/Port already exists in member"),
                               "ipaddress,port",
                               ipaddress + "," + port)
   
      #######################################################################
      ##########################   2. WARNINGS   ############################
      #######################################################################
   
      ### Validate 'Network' options submitted ###
      if options.has_key("ipaddress"):
         if not cluPing(ipaddress):
            raise OptionError("WARNING",
                              _("Powercontroller IP Address not found on network"),
                              "ipaddress",
                              ipaddress)

   ###########################################################
   # validate: will be called by outside apps.  validate is
   #           reponsible for validating all powercontroller
   #           options.
   ###########################################################
   def validate(self, options):
      if self.action == "add":
         ### Make sure option 'type' is submitted
         if not options.has_key("type"):
            raise OptionError("FATAL",
                              _("Type is required when adding a powercontroller"),
                              "type",
                              None)

         new_type = options["type"]
         new_general_type = self.get_general_type(new_type)
      else:
         ### Must be "modify"
         current_type = self.get("type")
         current_general_type = self.get_general_type(current_type)
         if options.has_key("type"):
            new_type = options["type"]
            new_general_type = self.get_general_type(new_type)
            if new_general_type == "error":
               raise OptionError("FATAL",
                                 _("Invalid Powercontroller Type"),
                                 "type",
                                 new_type)
         else:
            ### option "type" not submitted
            new_type = current_type
            new_general_type = current_general_type

      ### 'genericOptionChecks' will raise an error if either the data
      ### doesn't match the datatype(str or int), or the option is
      ### not in the list of valid options for this class.
      ClusterEntity.genericOptionChecks(self, options)

      if new_general_type == 'serial':
         self.__validate_serial(options)  ### errors raise exception
      elif new_general_type == 'network':
         ### new_general_type must be 'network'
         self.__validate_network(options) ### errors raise exception
      elif new_general_type == 'gulm':
         ##Maybe put gfs pkg check here one day
         pass 
      else:
         raise OptionError("FATAL",
                           _("Invalid Powercontroller Type"),
                           "type",
                           new_type)
          
      return
