#!/usr/bin/python
from xml.dom import minidom
from clusterpkg.member_module import member
import clusterpkg.member_module
from ClusterList import *

MEMBER_INFO = clusterpkg.member_module.MY_INFO

MY_INFO = {
   "WHOAMI"          : "members",
      
   "OPTIONS"         : { },
     
   "ADD_GETOPTS"     : [],
   "DELETE_GETOPTS"  : [],
   "UPDATE_GETOPTS"  : [],
   "LISTALL_GETOPTS" : [ "members" ],
}

class members(ClusterList):
   def __init__(self, node):
      parent = None
      ClusterList.__init__(self, MY_INFO, node, parent)

   ###################################################################
   # get_list: The list returned by ClusterList.get_list contains
   # ClusterEntity's - It's not sufficient for each to be a
   # ClusterEntity since ClusterEntity's don't have access to the
   # "member" functions.  Each member in the list returned by
   # this "get_list" must be a *member* entity in order to have
   # access to the "member" functions.
   ###################################################################
   def get_list(self):
      memberList = []
      for member_entity in ClusterList.get_list(self, MEMBER_INFO):
         memberList.append(member(member_entity.node, self))
      return memberList

   def get_member(self, val, key="name"):
      search_options = {key : val}
      member_entity = ClusterList.get_child(self, MEMBER_INFO, search_options)
      return member(member_entity.node, self)

   def member_found(self, search_options, member_being_validated=None):
      ### "member_being_validated" is either the member being modified, or
      ### the new member just added.  We want to be sure his key info doesn't
      ### duplicate an existing member.
      return ClusterList.child_found(self, MEMBER_INFO, search_options, member_being_validated)

   def add_member(self, cmdline_args=None):
      entity = ClusterList.add_child(self, MEMBER_INFO)
      member_entity = member(entity.node, self, "add")
      member_entity.add_options(cmdline_args)
      return member_entity

   def member_found_on_failoverdomain(self, name):
      ### Search all the nodes in all the failover domains for a name that
      ### matches the name of the member passed in.
      ownerDocument = self.node.ownerDocument
      failoverdomains_node = ownerDocument.getElementsByTagName("failoverdomains")[0]
      from clusterpkg.failoverdomains_module import failoverdomains
      failoverdomains_entity = failoverdomains(failoverdomains_node)
      for failoverdomain in failoverdomains_entity.get_list():
         nodes_in_domain = failoverdomain.get_failoverdomainnodes()
         for domain_node in nodes_in_domain:
            if domain_node.get("name") == name:
               return TRUE
      return FALSE

   def delete_member(self, name, cmdline_args=None):
      if self.member_found_on_failoverdomain(name):
         raise OptionError("FATAL",
                           _("Member delete not allowed - Member Name '%s' is specified by at least one failover domain") % name,
                           "name",
                           name)
      search_options = {"name": name}
      ClusterList.delete_child(self, MEMBER_INFO, search_options, cmdline_args)

   def delete_member_by_id(self, id):
      member = self.get_member(id, "id")
      name = member.get("name")
      if self.member_found_on_failoverdomain(name):
         raise OptionError("FATAL",
                           _("Member delete not allowed - Member Name '%s' is specified by at least one failover domain") % name,
                           "name",
                           name)
      ClusterList.delete_by_id(self, MEMBER_INFO, id)

   def out(self):
      nodeList = self.node.getElementsByTagName(MEMBER_INFO['WHOAMI'])
      for n in nodeList:
         member(n, self).out()
