#!/usr/bin/python
from ClusterEntity import *
import os, stat, string

TRUE = 1
FALSE = 0

ALLOW_SFS = FALSE # set to TRUE if you want to allow SFS

### DRIVERS key   = shared state 'type'
###         value = shared state 'driver' for that 'type'

VALID_SHAREDSTATE_TYPES = [ "raw" ]
if ALLOW_SFS:
   VALID_SHAREDSTATE_TYPES.append("sfs")

DRIVERS = { "raw" : "libsharedraw.so",
            "sfs" : "libsfs.so" }

MY_INFO = {
   "WHOAMI"  : "sharedstate",

   "OPTIONS" : {
      "type"       : { "datatype" : "str",
                       "order"    : 0,
                       "default"  : "raw" },

      "driver"     : { "datatype" : "str",
                       "order"    : 1,
                       "default"  : "libsharedraw.so" },

      "rawprimary" : { "datatype" : "str",
                       "order"    : 2,
                       "default"  : "/dev/raw/raw1" },

      "rawshadow"  : { "datatype" : "str",
                       "order"    : 3,
                       "default"  : "/dev/raw/raw2" },

      "sfspath"    : { "datatype" : "str",
                       "order"    : 4 },
   },

   "ADD_GETOPTS"     : [],

   "UPDATE_GETOPTS"  : [ "sharedstate",
                            "type=",
                            "driver=",
                            "rawprimary=",
                            "rawshadow=",
                            "sfspath=" ],

   "DELETE_GETOPTS"  : [],
   "LISTALL_GETOPTS" : [],
}

class sharedstate(ClusterEntity):
   def __init__(self, node):
      ClusterEntity.__init__(self, MY_INFO, node)

   def __validate_raw_device(self, raw_type, option, value):
      ### Use os.path.exists(value), or
      ###     os.stat(value) - os.stat(value) will raise OSError if not a file.
      try:
         mode = os.stat(value)[stat.ST_MODE]
      except OSError, e:
         #print "OSError - dir(e) = %s" % dir(e)
         #print "  OSError - e.args     =", e.args
         #print "  OSError - e.errno    = %s" % e.errno
         #print "  OSError - e.filename = %s" % e.filename
         #print "  OSError - e.strerror = %s" % e.strerror
         msg = _("Raw %s device does not exist") % raw_type
         raise OptionError('FATAL', msg, option, value)

      if not stat.S_ISCHR(mode):
         msg = _("Raw %s device is not a character special file") % raw_type
         raise OptionError('FATAL', msg, option, value)

   def __validate_sfspath(self, sfspath):
      try:
         mode = os.stat(sfspath)[stat.ST_MODE]
      except OSError, e:
         msg = _("SFS Path does not exist")
         raise OptionError("FATAL", msg, "sfspath", sfspath)

      if not stat.S_ISDIR(mode):
         msg = _("SFS Path must be a directory")
         raise OptionError("FATAL", msg, "sfspath", sfspath)

###########################################################################

   def __validate_raw(self, options):

      ### if any 'sfs' options were submitted, then error ###

      ### From GUI, raw and sfs options will all be submitted, but
      ### sfs options should have null("") value.
      if options.has_key("sfspath") and options["sfspath"] != "":
         raise OptionError("FATAL",
                           _("SFS Path is an invalid option for a Raw Shared State"),
                           "sfspath",
                           options["sfspath"])

      ### validate 'raw' fields - and assign option to 'options' dictionary
      ### if option does NOT currently exist, and there IS a default value
      rawprimary = ""
      rawshadow  = ""

      if options.has_key("rawprimary") and options["rawprimary"] != "":
         self.__validate_raw_device("Primary",
                                    "rawprimary",
                                    options["rawprimary"])
         rawprimary = options["rawprimary"]
      else:
         raise OptionError("FATAL",
                           _("Raw Primary device is required for Raw Shared State"),
                           "rawprimary",
                           None)

      if options.has_key("rawshadow") and options["rawshadow"] != "":
         self.__validate_raw_device("Shadow",
                                    "rawshadow",
                                    options["rawshadow"])
         rawshadow = options["rawshadow"]
      else:
         raise OptionError("FATAL",
                           _("Raw Shadow device is required for Raw Shared State"),
                           "rawshadow",
                           None)

      if options.has_key("driver") and options["driver"] != "":
         pass
      elif not self.get("driver") and \
         MY_INFO["OPTIONS"]["driver"].has_key("default"):
            ### 'driver' was not submitted, and there was no existing
            ### 'driver', and there is a default, so assign the default
            options["driver"] = MY_INFO["OPTIONS"]["driver"]["default"]

      #############################
      # validation between fields #
      #############################
      if rawprimary == "":
         ### rawprimary NOT submitted - see if that option already exists
         ### in the sharedstate tag
         try:
            rawprimary = self.get("rawprimary")
         except OptionError, e:
            ### rawprimary was not submitted, nor does it already exist
            ### in the sharedstate tag - error.
            raise OptionError("FATAL",
                              _("Primary Quorum Partition is required"),
                              "rawprimary",
                              None)

      if rawshadow == "":
         ### rawshadow NOT submitted - see if that option already exists
         ### in the sharedstate tag
         try:
            rawshadow = self.get("rawshadow")
         except OptionError, e:
            ### rawshadow was not submitted, nor does it already exist
            ### in the sharedstate tag - error.
            raise OptionError("FATAL",
                              _("Shadow Quorum Partition is required"),
                              "rawshadow",
                              None)

      if rawprimary == rawshadow:
         raise OptionError("FATAL",
                           _("Primary Quorum Partition must not be the same as the Shadow Quorum Partition"),
                           None,
                           None)

   def __validate_sfs(self, options):

      ### if any 'raw' options were submitted, then error ###

      ### From GUI, raw and sfs options will all be submitted, but
      ### raw options should have null("") value.
      if options.has_key("rawprimary") and options["rawprimary"] != "":
         raise OptionError("FATAL",
                           _("Raw Primary Device is an invalid option for an SFS Shared State"),
                           "rawprimary",
                           options["rawprimary"])

      if options.has_key("rawshadow") and options["rawshadow"] != "":
         raise OptionError("FATAL",
                           _("Raw Shadow Device is an invalid option for an SFS Shared State"),
                           "rawshadow",
                           options["rawshadow"])

      ###
      ### sfspath is required to be present(existing or newly submitted) ###
      ###
      if options.has_key("sfspath"):
         if options["sfspath"] == "":
            raise OptionError("FATAL",
                              _("SFS Path is required"),
                              "sfspath",
                              None)

         self.__validate_sfspath(options["sfspath"])

      elif not self.get("sfspath"):
         ### 'sfspath' was not submitted, nor does it exist in
         ### the current 'sfs' tag - ERROR.
         raise OptionError("FATAL",
                           _("SFS Path is required"),
                           "sfspath",
                           None)

      if not options.has_key("driver") and not self.get("driver"):
         options["driver"] = DRIVERS["sfs"]

###########################################################################

   def validate(self, options_orig):
      options = options_orig.copy()
      if len(options) == 0:
         return

      if self.action == "add":
         ### Make sure option 'type' is submitted
         if not options.has_key("type"):
            raise OptionError("FATAL",
                              _("Type is required when adding Shared State"),
                              "type",
                              None)
         type = options["type"]
      else:
         ### Must be "modify"
         prev_type = self.get("type")
         if options.has_key("type"):
            type = options["type"]
            if not type in VALID_SHAREDSTATE_TYPES:
               if len(VALID_SHAREDSTATE_TYPES) == 1:
                  msg = "Type must be '%s'" % VALID_SHAREDSTATE_TYPES[0]
               elif len(VALID_SHAREDSTATE_TYPES) == 2:
                  msg = "Type must be either '%s' or '%s'" % (VALID_SHAREDSTATE_TYPES[0], VALID_SHAREDSTATE_TYPES[1])
               raise OptionError("FATAL",
                                 _(msg),
                                 "type",
                                 type)
         else:
            type = prev_type

      ### 'genericOptionChecks' will raise an error if either the data
      ### doesn't match the datatype(str or int), or the option is
      ### not in the list of valid options for this class.
      ClusterEntity.genericOptionChecks(self, options)

      if type == "raw":
         self.__validate_raw(options)
      else:
         self.__validate_sfs(options)
