#!/usr/bin/python

import re
import string
from clusterpkg.cluster_functions import getStatusOutput
 
INET_ADDR_PATTERN       = re.compile('^\s*inet addr')
START_INTERFACE_PATTERN = re.compile('^[\w|:]+')

IFCONFIG_CMD = "LC_ALL=C /sbin/ifconfig" # need LC_ALL=C so different
                                         # languages don't produce
                                         # different ifconfig output

TRUE = (1==1)
FALSE = not TRUE
 
class ifConfig:
    """Parses 'ifconfig' output and makes info available. """
    def __init__(self):
        (status, output) = getStatusOutput(IFCONFIG_CMD)
        self.ifConfig = {} # create dictionary
        for line in output.splitlines():
            if line == "":
                continue # skip blank lines

            if START_INTERFACE_PATTERN.search(line):
                self.__parse_interface(line)
            elif INET_ADDR_PATTERN.search(line):
                self.__parse_inet_addr(line)

        #for iface_key, iface_dict in self.ifConfig.items():
        #    print "\nInterface:", iface_key
        #    for key, value in iface_dict.items():
        #        print "  key=[%s], value=[%s]" % (key, value)

    def __parse_interface(self, line):
        match_obj = re.search(START_INTERFACE_PATTERN, line)
        iface = match_obj.group()
        self.__current_interface = iface
        self.ifConfig[iface] = {}

    def __parse_inet_addr(self, line):
        rest_line = string.lstrip(line) # discard spaces at the start
        tokens = rest_line.split("  ")  # split on 2 spaces
        for token in tokens:
           key, value = token.split(":")
           self.ifConfig[self.__current_interface][key] = value

    def getAllInterfaceNames(self, INCLUDE_LO=TRUE):
        interfaces = []
        for interface in self.ifConfig.keys():
            if interface == "lo" and not INCLUDE_LO:
                continue
            interfaces.append(interface)
        return interfaces

    def getInetAddr(self, interface):
        if self.ifConfig[interface].has_key("inet addr"):
            return self.ifConfig[interface]['inet addr']
        return None

    def getNetmask(self, interface):
        if self.ifConfig[interface].has_key("Mask"):
            return self.ifConfig[interface]['Mask']
        return None

    def getBroadcast(self, interface):
        if self.ifConfig[interface].has_key("Bcast"):
            return self.ifConfig[interface]['Bcast']
        return None
