#!/usr/bin/python

## deviceDialog.py

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import gtk
import gtk.glade
import gettext
import os, string

from clusterpkg.cluster_functions     import OptionError
from clusterpkg.cluster_functions_gui import errorMessage, warningMessage
from clusterpkg.util_smbconf_module   import smbConf

import nfsexportDialog

TRUE = (1==1)
FALSE = not TRUE

##
## I18N
##

class deviceDialog: #(gtk.Dialog):
    def __init__(self, glade_xml, device):
        #gtk.Dialog.__init__(self, "")
        self.glade_xml = glade_xml
        self.device = device

        self.dialog = glade_xml.get_widget('deviceDialog')

        self.refresh = TRUE

    def on_cancelbutton_clicked(self, button):
        self.refresh = TRUE

    def on_okbutton_clicked(self, button):

        ### "get" the new values from the Gui widgets ###

        name = self.glade_xml.get_widget('device_name_entry').get_text()
        sharename = self.glade_xml.get_widget('device_sharename_entry').get_text()
        mountpoint = self.glade_xml.get_widget('mount_mountpoint_entry').get_text()
        fstype = self.glade_xml.get_widget('mount_fstype_combo-entry').get_text()
        options = self.glade_xml.get_widget('mount_options_entry').get_text()
        forceunmount = self.glade_xml.get_widget('mount_forceunmount_checkbutton').get_active()
        if forceunmount == TRUE:
            forceunmount = "yes"
        else:
            forceunmount = "no"

        self.refresh = TRUE

        ### If user didn't update any fields, then just return.
        if name         == self.name       and \
           sharename    == self.sharename  and \
           mountpoint   == self.mountpoint and \
           fstype       == self.fstype     and \
           options      == self.options    and \
           forceunmount == self.forceunmount:
              self.dialog.response(gtk.RESPONSE_OK)
              return

        ### Validate *Device* options first ###
        ###    Although the main 'mountpoint' validation happens below
        ###    in self.check("mount", validate_options), we also need
        ###    to pass 'mountpoint' in as an option to 'device' - device
        ###    validation checks 'sharename' - if a sharename is specified,
        ###    the samba share config file needs a 'path'(mounpoint), so
        ###    'device' validation needs to ensure that a mountpoint exists.
        #validate_options = { "name"      : name,
        #                     "sharename" : sharename }
        validate_options = { "name"      : name,
                             "sharename" : sharename,
                             "mountpoint" : mountpoint }
        rc = self.check("device", validate_options)
        if rc != gtk.RESPONSE_OK:
            return

        ### Validate *Mount* options next ###
        validate_options = { "mountpoint"   : mountpoint,
                             "fstype"       : fstype,
                             "options"      : options,
                             "forceunmount" : forceunmount }
        rc = self.check("mount", validate_options)
        if rc != gtk.RESPONSE_OK:
            return

        ### Validation passed - now 'set' DOM XML options ###
        self.device.set("name", name)
        self.device.set("sharename", sharename)
        self.mount.set("mountpoint", mountpoint)
        self.mount.set("fstype", fstype)
        self.mount.set("options", options)
        self.mount.set("forceunmount", forceunmount)

        self.device.change_action_to_modify()
        return

    def check(self, check_type, validate_options):
        rc = gtk.RESPONSE_OK
        try:
            ### Invoke device or mount (both are really just ClusterEntity's)
            ### "check" function to validate option.
            if check_type == "device":
                self.device.validate(validate_options)
            else:
                self.mount.validate(validate_options)
        except OptionError, e:
            if e.getSeverity() == 'FATAL':
                errorMessage(e.getMessage())
                self.refresh = FALSE # Don't refresh - leave bad values
                                     # on screen.
                rc = gtk.RESPONSE_REJECT

            elif e.getSeverity() == 'WARNING':
                msg = _("%s\n\nPress 'OK' to Continue, or  'Cancel' to Exit.") % e.getMessage()
                rc = warningMessage(msg)
                if rc != gtk.RESPONSE_OK:
                   self.refresh = FALSE     # Don't refresh - leave bad values
                                            # on screen.
                   rc = gtk.RESPONSE_REJECT # Need to differentiate this
                                            # 'Cancel' from the main Dialog
                                            # 'Cancel', so make this rc
                                            # gtk.RESPONSE_REJECT.

        self.dialog.response(rc)
        return rc

    def __get_fstypes_list(self):
        fd = open('/proc/filesystems', 'r')
        fstypes_list = ["None"]
        for line in fd.readlines():
            line = string.strip(line)
            fields = string.split(line)
            fstype = fields[0]
            if fstype != "nodev":
                fstypes_list.append(fstype)

        fd.close()
        fstypes_list.sort()
        return fstypes_list

    def run(self, device, action):
        self.device = device
        self.action = action
        self.mount  = device.get_mount()

        ### Now connect the buttons to the functions ###
        self.ok     = self.glade_xml.get_widget('deviceDialog_okbutton').connect("clicked", self.on_okbutton_clicked)
        self.cancel = self.glade_xml.get_widget('deviceDialog_cancelbutton').connect("clicked", self.on_cancelbutton_clicked)

        ### Load screen widgets with data from XML DOM for selected device ###

        if self.refresh:
            self.name = device.get("name")
            self.glade_xml.get_widget('device_name_entry').set_text(self.name)

            self.sharename = device.get("sharename")
            self.glade_xml.get_widget('device_sharename_entry').set_text(self.sharename)

            self.mountpoint = self.mount.get("mountpoint")
            self.glade_xml.get_widget('mount_mountpoint_entry').set_text(self.mountpoint)

            mount_fstype_combo_widget = self.glade_xml.get_widget('mount_fstype_combo')
            fstypes_list = self.__get_fstypes_list()
            mount_fstype_combo_widget.set_popdown_strings(fstypes_list)

            self.fstype = self.mount.get("fstype")
            mount_fstype_combo_entry = self.glade_xml.get_widget('mount_fstype_combo-entry')
            mount_fstype_combo_entry.set_text(self.fstype)

            self.options = self.mount.get("options")
            self.glade_xml.get_widget('mount_options_entry').set_text(self.options)

            self.forceunmount = self.mount.get("forceunmount")
            if self.forceunmount in ("no", "n", "No", "N"):
                clu_xml_value = FALSE
            else:
                clu_xml_value = TRUE
            self.glade_xml.get_widget('mount_forceunmount_checkbutton').set_active(clu_xml_value)

        rc = self.dialog.run()
        self.glade_xml.get_widget('deviceDialog_okbutton').disconnect(self.ok)
        self.glade_xml.get_widget('deviceDialog_cancelbutton').disconnect(self.cancel)
        return rc

    def hide(self):
        return self.dialog.hide()

class deviceItem:
    def __init__(self, parent, device):
        ### parent is a "serviceItem" instance ###
        self.parent    = parent
        self.glade_xml = parent.glade_xml
        self.treeview  = parent.treeview
        self.model     = parent.treeview.get_model()
        self.columns   = parent.columns
        self.device    = device

        self.dialog = deviceDialog(self.glade_xml, device)

    def GUI_load_row_columns(self, device_item, iter):
        row_display_string = _("%s  <device>") % device_item.device.get("name")
        self.model.set(iter,
                       self.columns['COLUMN_KEY'], row_display_string,
                       self.columns['COLUMN_DATA'], device_item)

    def GUI_tree_load_device(self, iter):
        device_iter = self.model.append(iter)
        self.GUI_load_row_columns(self, device_iter)

        for n in self.device.get_nfsexports():
            nfsexport_item = nfsexportDialog.nfsexportItem(self, n)
            nfsexport_item.GUI_tree_load_nfsexport(device_iter)

    def get_type(self):
        return "device"

    def get_value_of_key_field(self):
        return self.get("name") # Device key field is "name"

    def get(self, option):
        return self.device.get(option)

    def __modify(self, iter, action="modify"):
        if action == "modify":
            device = self.device
            device_item = self
        else:
            ### Must be an "add" ###
            service_item = self.parent
            service_entity = service_item.service

            device = service_entity.add_device()
            device_item = deviceItem(self.parent, device)

        while 1:
            rc = self.dialog.run(device, action)
            if rc == gtk.RESPONSE_OK:
                if action == "add":
                    ### model.append needs the "iter" of the *parent* of the
                    ### thing that is being appended.
                    parent_iter = iter
                    iter = self.model.append(parent_iter)

                self.GUI_load_row_columns(device_item, iter)

            elif rc == gtk.RESPONSE_CANCEL:
                if action == "add":
                    service_entity.delete_device_by_id(device.get("id"))
            else:
                ### rc was gtk.RESPONSE_REJECT:
                ###   1. User got a FATAL error and clicked 'OK' (only option)
                ###      so display same dialog again,
                ###        OR
                ###   2. User got a WARNING and clicked 'Cancel' to go back
                ###      and re-enter.
                continue
            break

        self.dialog.hide()
                
    def add(self, parent_iter):
        self.__modify(parent_iter, "add")

    def clear_failed_exports(self):
        self.__failed_exports = []

    def get_failed_exports(self):
        return self.__failed_exports

    def add_export(self, iter,export,hosts):
        nfsexport = self.device.add_nfsexport()
        options = { "name" : export }
        try:
            nfsexport.validate(options)
        except OptionError, e:
            clients_list = []
            for host in hosts:
               client = host.getHosts()
               options = host.getOptions()
               client_string = "%s(%s)" % (client, options)
               clients_list.append(client_string)
               clients_string = string.join(clients_list, " ")
               export_string = "%s %s\n***ERROR: %s\n" % (export, clients_string, e.getMessage())
               self.__failed_exports.append(export_string)
               self.device.delete_nfsexport_by_id(nfsexport.get("id"))
               return None

        nfsexport_item = nfsexportDialog.nfsexportItem(self, nfsexport)
        iter = self.model.append(iter)
        nfsexport.set("name",export)
        nfsexport_item.GUI_load_row_columns(nfsexport_item, iter)
        for h in hosts:
            nfsexport_item.add_host(iter,h)

    def add_child(self, iter):
        nfsexport = self.device.add_nfsexport()
        nfsexport_item = nfsexportDialog.nfsexportItem(self, nfsexport)
        nfsexport_dialog = nfsexportDialog.nfsexportDialog(self.glade_xml, nfsexport)
        action = "add"
        while 1:
            rc = nfsexport_dialog.run(nfsexport, action)
            if rc == gtk.RESPONSE_OK:
                iter = self.model.append(iter)

                nfsexport_item.GUI_load_row_columns(nfsexport_item, iter)
                break
            if rc == gtk.RESPONSE_CANCEL:
                self.device.delete_nfsexport_by_id(nfsexport.get("id"))
                break

        nfsexport_dialog.hide()

    def properties(self, iter):
        self.__modify(iter)

    def delete(self):
        service_item = self.parent
        service_entity = service_item.service
        service_entity.delete_device_by_id(self.device.get("id"))
