#!/usr/bin/python

## failoverdomainDialog.py

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import gtk
import gtk.glade
import gettext
from string import strip

from clusterpkg.cluster_functions import OptionError
from clusterpkg.cluster_functions_gui import errorMessage, warningMessage

import failoverdomainnodeDialog
import failoverdomainnodesListview

from clusterpkg.services_module import services

##
## I18N
##

TRUE = (1==1)
FALSE = not TRUE

NODE_COLUMN_NAME = 0
NODE_COLUMN_DATA = 1

class failoverdomainDialog: #gtk.Dialog):
    def __init__(self, glade_xml):
        #print "failoverdomainDialog __init__ Now!"
        self.glade_xml = glade_xml

        self.dialog = glade_xml.get_widget('failoverdomainDialog')

        self.failoverdomainnodesList = failoverdomainnodesListview.failoverdomainnodesListview(self.glade_xml)

        self.refresh = TRUE

    def on_cancelbutton_clicked(self, button):
        # Reset GUI widget values back to what they were - in this dialog
        # one or more widgets could have been set to illegal values, which
        # they would still have if we didn't reset them here.
        self.refresh = TRUE

    def on_okbutton_clicked(self, button):

        ### "get" the new values from the Gui widgets ###
        name = strip(self.glade_xml.get_widget('failoverdomain_name_entry').get_text())

        restricted = self.glade_xml.get_widget('failoverdomain_restricted_checkbutton').get_active()
        if restricted == TRUE:
            restricted = "yes"
        else:
            restricted = "no"

        ordered = self.glade_xml.get_widget('failoverdomain_ordered_checkbutton').get_active()
        if ordered == TRUE:
            ordered = "yes"
        else:
            ordered = "no"

        ### Edits for each *individual* field exist in that fields'
        ### "validate" function which will be invoked by the "set"
        ### in the "try" below.

        ### Call 'validate' with all options ###
        options = { "name"       : name,
                    "restricted" : restricted,
                    "ordered"    : ordered }

        self.refresh = TRUE
        rc = gtk.RESPONSE_OK
        try:
            ### Invoke failoverdomain 'validate' to validate all options
            self.failoverdomain.validate(options)
        except OptionError, e:
            if e.getSeverity() == 'FATAL':
                errorMessage(e.getMessage())
                self.refresh = FALSE # Don't refresh - leave bad values
                                     # on screen.
                rc = gtk.RESPONSE_REJECT

            elif e.getSeverity() == 'WARNING':
                msg = _("%s\n\nPress 'OK' to Continue, or  'Cancel' to Exit.") % e.getMessage()
                rc = warningMessage(msg)
                if rc != gtk.RESPONSE_OK:
                   self.refresh = FALSE     # Don't refresh - leave bad values
                                            # on screen.
                   rc = gtk.RESPONSE_REJECT # Need to differentiate this
                                            # 'Cancel' from the main Dialog
                                            # 'Cancel', so make this rc
                                            # gtk.RESPONSE_REJECT.

        if rc != gtk.RESPONSE_OK:
            self.dialog.response(rc)
            return
        
        ### Validation passed - now 'set' DOM XML options ###
        self.failoverdomain.set("name", name)
        self.failoverdomain.set("restricted", restricted)
        self.failoverdomain.set("ordered", ordered)

        ############# commented out 7/17/2003 HM #######################
        ### Command line tool needed to do this same thing, so instead
        ### of duplicating the code there, in failoverdomain_module.py
        ### I
        ###   1. created new function 'set' that overrides the
        ###      ClusterEntity 'set', and
        ###   2. moved this logic into that new 'set' function.
        ### so now when failoverdomain.set is called to change the
        ### "name" option, all services that referred to the
        ### "old" failoverdomain name will automatically be changed
        ### to refer to the "new" failoverdomain name.
        ################################################################
        ### If domain name changed, then change all services that
        ### refer to the old name - make them refer to the new name
        #if self.name != "":
        #    old_failoverdomain = self.name
        #    new_failoverdomain = name
        #    self.failoverdomain.change_failoverdomain_on_services(old_failoverdomain, new_failoverdomain)

        ### Remove all existing domain *nodes*, and then add all new
        ### domain nodes.
        existing_domain_nodes = self.failoverdomain.get_failoverdomainnodes()
        for n in existing_domain_nodes:
            self.failoverdomain.delete_failoverdomainnode_by_id(n.get("id"))

        new_domain_nodes = self.failoverdomainnodesList.get_nodes_in_domain() 
        ct = 0
        for n in new_domain_nodes:
            new_domain_node = self.failoverdomain.add_failoverdomainnode()
            new_domain_node.set("name", n.get("name"))
            ct = ct + 1

        self.failoverdomain.change_action_to_modify()
        self.dialog.response(rc)
        return

    def on_add_node_button_clicked(self, button):
        self.failoverdomainnodesList.add()
        return

    def on_delete_node_button_clicked(self, button):
        self.failoverdomainnodesList.delete()

    def run(self, failoverdomain, action):
        self.failoverdomain = failoverdomain
        self.action = action
        ### Since failoverdomain couldn't be used in __init__ instantiation
        ### of failoverdomainnodesList, call to set it now before it is
        ### needed in on_add_node_button_clicked.
        self.failoverdomainnodesList.set_failoverdomain(failoverdomain)

        ### Now connect the buttons to the functions ###
        self.add_node_button = self.glade_xml.get_widget('failoverdomainDialog_add_node_button').connect("clicked", self.on_add_node_button_clicked)
        self.delete_node_button = self.glade_xml.get_widget('failoverdomainDialog_delete_node_button').connect("clicked", self.on_delete_node_button_clicked)
        self.ok = self.glade_xml.get_widget('failoverdomainDialog_okbutton').connect("clicked", self.on_okbutton_clicked)
        self.cancel = self.glade_xml.get_widget('failoverdomainDialog_cancelbutton').connect("clicked", self.on_cancelbutton_clicked)

        ### Populate the screen GUI widgets with values from XML DOM ###

        if self.refresh:
            self.name = failoverdomain.get("name")
            self.glade_xml.get_widget('failoverdomain_name_entry').set_text(self.name)

            self.restricted = failoverdomain.get("restricted")
            if self.restricted == "yes":
                gui_value = TRUE
            else:
                gui_value = FALSE
            self.glade_xml.get_widget('failoverdomain_restricted_checkbutton').set_active(gui_value)

            self.ordered = failoverdomain.get("ordered")
            if self.ordered == "yes":
                gui_value = TRUE
            else:
                gui_value = FALSE
            self.glade_xml.get_widget('failoverdomain_ordered_checkbutton').set_active(gui_value)

            failoverdomainnode_entities = failoverdomain.get_failoverdomainnodes()
            self.failoverdomainnodesList.set_nodes_in_domain(failoverdomain, failoverdomainnode_entities)

        rc = self.dialog.run()
        self.glade_xml.get_widget('failoverdomainDialog_add_node_button').disconnect(self.add_node_button)
        self.glade_xml.get_widget('failoverdomainDialog_delete_node_button').disconnect(self.delete_node_button)
        self.glade_xml.get_widget('failoverdomainDialog_okbutton').disconnect(self.ok)
        self.glade_xml.get_widget('failoverdomainDialog_cancelbutton').disconnect(self.cancel)
        return rc

    def hide(self):
        return self.dialog.hide()

class failoverdomainItem:
    def __init__(self, parent, failoverdomain):
        ### parent is "failoverdomainsTreeview" instance ###
        self.parent    = parent
        self.glade_xml = parent.glade_xml
        self.tree_view = parent.tree_view
        self.model     = parent.tree_view.get_model()
        self.columns   = parent.columns
        self.failoverdomain = failoverdomain
        self.dialog    = parent.dialog

    def GUI_load_row_columns(self, failoverdomain_item, iter):
        row_display_string = failoverdomain_item.failoverdomain.get("name")
        self.model.set(iter,
                       self.columns['COLUMN_KEY'], row_display_string,
                       self.columns['COLUMN_DATA'], failoverdomain_item)

    def GUI_list_load_failoverdomain(self):
        iter = self.model.append()
        self.GUI_load_row_columns(self, iter)

    def get_type(self):
        return "failoverdomain"

    def get_value_of_key_field(self):
        return self.get("name")

    def get(self, option):
        return self.failoverdomain.get(option)

    def __modify(self, iter, action="modify"):
        if action == "modify":
            failoverdomain = self.failoverdomain
            failoverdomain_item = self
        else:
            ### Must be an "add" ###
            failoverdomains_entity = self.parent.failoverdomains_entity
            failoverdomain = failoverdomains_entity.add_failoverdomain()
            failoverdomain_item = failoverdomainItem(self.parent, failoverdomain)

        while 1:
            rc = self.dialog.run(failoverdomain, action)
            if rc == gtk.RESPONSE_OK:
                if action == "add":
                    ### ListStore 'append' doesn't take any arguments.
                    iter = self.model.append()

                self.GUI_load_row_columns(failoverdomain_item, iter)

            elif rc == gtk.RESPONSE_CANCEL:
                if action == "add":
                    failoverdomains_entity.delete_failoverdomain_by_id(failoverdomain.get("id"))
            else:
                ### rc was gtk.RESPONSE_REJECT:
                ###   1. User got a FATAL error and clicked 'OK' (only option)
                ###      so display same dialog again,
                ###        OR
                ###   2. User got a WARNING and clicked 'Cancel' to go back
                ###      and re-enter.
                continue
            break

        self.dialog.hide()

    def add(self, iter):
        self.__modify(iter, "add")

    def properties(self, iter):
        self.__modify(iter)

    ### delete a 'domain' ###
    def delete(self):
        ### self.parent is a 'failoverdomainsTreeview' ###
        failoverdomains_entity = self.parent.failoverdomains_entity
        try:
            failoverdomains_entity.delete_failoverdomain(self.failoverdomain.get("name"))
        except OptionError, e:
            errorMessage(e.getMessage())
            return FALSE # delete failed
        return TRUE      # delete succeeded
