#!/usr/bin/python

## failoverdomainnodeDialog.py

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import gtk
import gtk.glade
import gettext
import gobject

from clusterpkg.cluster_functions     import OptionError
from clusterpkg.cluster_functions_gui import errorMessage

from clusterpkg.members_module import members

##
## I18N
##

COLUMN_NAME = 0 # 0 for 1st column.

TRUE = (1==1)
FALSE = not TRUE

class failoverdomainnodeDialog:
    def __init__(self, glade_xml):
        self.glade_xml = glade_xml

        self.tree_view = self.glade_xml.get_widget ('failoverdomain_selectnodes_treeview')
        ### Just a simple list - no children, so use 'ListStore' (not TreeStore)
        self.model = gtk.ListStore(gobject.TYPE_STRING,
                                   gobject.TYPE_PYOBJECT)
        self.tree_view.set_model(self.model)

        column = gtk.TreeViewColumn(_("Name"),
                                    gtk.CellRendererText(),
                                    text=COLUMN_NAME)
        self.tree_view.append_column(column)

        ### set up the treeview to do multiple selection ###
        self.tree_view.get_selection().set_mode(gtk.SELECTION_MULTIPLE)

        self.dialog = glade_xml.get_widget('failoverdomainnodeDialog')

        self.refresh = TRUE

        self.selected_node_names = []

    def on_cancelbutton_clicked(self, button):
        self.refresh = TRUE

    def get_selected_node_names(self):
        return self.selected_node_names

    def on_okbutton_clicked(self, button):

        ### "get" the new values from the Gui widgets ###

        node_names_selected = []
        def foreach_func (model, path, iter, text):
            node_names_selected.append(model.get_value (iter, COLUMN_NAME))
        selection = self.tree_view.get_selection()
        selection.selected_foreach(foreach_func, None)
        self.selected_node_names = node_names_selected
        return

    def run(self, failoverdomainnode_item):
        failoverdomainnode = failoverdomainnode_item.failoverdomainnode
        parent = failoverdomainnode_item.parent
        failoverdomain_entity = parent.failoverdomain_entity

        ### Now connect the buttons to the functions ###
        self.ok = self.glade_xml.get_widget('failoverdomainnodeDialog_okbutton').connect("clicked", self.on_okbutton_clicked)
        self.cancel = self.glade_xml.get_widget('failoverdomainnodeDialog_cancelbutton').connect("clicked", self.on_cancelbutton_clicked)

        ### Populate the screen widgets ###
        if self.refresh:

            ### find members currently in XML (in memory) ###

            ### To get current list of members, need to get DOM XML
            ### ownerDocument(root DOM XML node), so we can then
            ### find(using getElementsByTagName) the 'members' node.
            ownerDocument = failoverdomain_entity.node.ownerDocument
            members_node = ownerDocument.getElementsByTagName("members")[0]
            members_entity = members(members_node)

            ### need this model.clear ###
            self.model.clear()
            nodes_in_domain = failoverdomain_entity.get_failoverdomainnodes()
            for m in members_entity.get_list():
                member_name = m.get("name")

                ### Only load nodes to the list screen that were not
                ### previously selected.
                found = FALSE
                for n in nodes_in_domain:
                    if n.get("name") == member_name:
                        found = TRUE
                        break
                if not found:
                    iter = self.model.append()
                    self.model.set(iter,
                                   COLUMN_NAME, member_name)

        rc = self.dialog.run()
        self.glade_xml.get_widget('failoverdomainnodeDialog_okbutton').disconnect(self.ok)
        self.glade_xml.get_widget('failoverdomainnodeDialog_cancelbutton').disconnect(self.cancel)
        return rc

    def hide(self):
        return self.dialog.hide()

class failoverdomainnodeItem:
    def __init__(self, parent, failoverdomainnode):
        ### parent is failoverdomainnodesListview object ###
        self.parent    = parent
        self.glade_xml = parent.glade_xml
        self.failoverdomainnode = failoverdomainnode

        self.dialog = parent.dialog

    def get_type(self):
        return "failoverdomainnode"

    def get_value_of_key_field(self):
        return self.get("name") # failoverdomainnode key field is "name"

    def get(self, option):
        return self.failoverdomainnode.get(option)

    def __modify(self, action="modify"):
        while 1:
            rc = self.dialog.run(self)
            self.dialog.hide()
            return rc

    def add(self):
        rc = self.__modify("add")
        if rc == gtk.RESPONSE_CANCEL:
            return []
        node_names_list = self.dialog.get_selected_node_names()
        new_node_items_list = []
        for node_name in node_names_list:
            failoverdomain_entity = self.parent.failoverdomain_entity
            new_failoverdomainnode = failoverdomain_entity.add_failoverdomainnode()
            new_failoverdomainnode.set("name", node_name)
            new_failoverdomainnode_item = failoverdomainnodeItem(self.parent, new_failoverdomainnode)
            new_node_items_list.append(new_failoverdomainnode_item)
        return new_node_items_list

    def delete(self):
        failoverdomain_entity = self.parent.failoverdomain_entity
        failoverdomain_entity.delete_failoverdomainnode_by_id(self.failoverdomainnode.get("id"))
