#!/usr/bin/python

## memberDialog.py

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import gtk
import gtk.glade
import gettext
from string import strip

from clusterpkg.cluster_functions import OptionError
from clusterpkg.cluster_functions_gui import errorMessage, warningMessage

import powercontrollerDialog

##
## I18N
##

TRUE = (1==1)
FALSE = not TRUE

class memberDialog: #gtk.Dialog):
    def __init__(self, glade_xml):
        self.glade_xml = glade_xml

        self.dialog = glade_xml.get_widget('memberDialog')

        self.refresh = TRUE

    def on_cancelbutton_clicked(self, button):
        # Reset GUI widget values back to what they were - in this dialog
        # one or more widgets could have been set to illegal values, which
        # they would still have if we didn't reset them here.
        self.refresh = TRUE

    def on_okbutton_clicked(self, button):

        ### "get" the new values from the Gui widgets ###
        name = strip(self.glade_xml.get_widget('member_name_entry').get_text())

        watchdog = self.glade_xml.get_widget('member_watchdog_checkbutton').get_active()
        if watchdog == TRUE:
            watchdog = "yes"
        else:
            watchdog = "no"

        options = { "name"     : name,
                    "watchdog" : watchdog }

        self.refresh = TRUE
        rc = gtk.RESPONSE_OK
        try:
            ### Invoke member 'validate' to validate all options
            self.member.validate(options)
        except OptionError, e:
            if e.getSeverity() == 'FATAL':
                #print "memberDialog check 'except': in FATAL now!"
                errorMessage(e.getMessage())
                self.refresh = FALSE # Don't refresh - leave bad values
                                         # on screen.
                self.dialog.response(gtk.RESPONSE_REJECT)
                rc = gtk.RESPONSE_REJECT

            elif e.getSeverity() == 'WARNING':
                msg = _("%s\n\nPress 'OK' to Continue, or 'Cancel' to Exit.") % e.getMessage()
                rc = warningMessage(msg)
                if rc != gtk.RESPONSE_OK:
                   self.refresh = FALSE # Don't refresh - leave bad values
                                            # on screen.
                   self.dialog.response(gtk.RESPONSE_REJECT)

        if rc != gtk.RESPONSE_OK:
            return
        
        ### Validation passed - now 'set' DOM XML options ###
        self.member.set("name", name)
        self.member.set("watchdog", watchdog)

        ############# commented out 7/17/2003 HM #######################
        ### Command line tool needed to do this same thing, so instead
        ### of duplicating the code there, in member_module.py I
        ###   1. created new function 'set' that overrides the
        ###      ClusterEntity 'set', and
        ###   2. moved this logic into that new 'set' function.
        ### so now when member.set is called to change the "name"
        ### option, all failoverdomainnodes that referred to the
        ### "old" member name will automatically be changed to the
        ### "new" member name.
        ################################################################
        ### If member name changed, then change all failoverdomainnodes that
        ### refer to the old name - make them refer to the new name.
        #if self.name != "":
        #    old_name = self.name
        #    new_name = name
        #    self.member.change_member_on_failoverdomainnodes(old_name, new_name)

        self.member.change_action_to_modify()
        return

    def run(self, member, action):
        self.member = member
        self.action = action

        ### Now connect the buttons to the functions ###
        self.ok = self.glade_xml.get_widget('memberDialog_okbutton').connect("clicked", self.on_okbutton_clicked)
        self.cancel = self.glade_xml.get_widget('memberDialog_cancelbutton').connect("clicked", self.on_cancelbutton_clicked)

        ### Populate the screen GUI widgets with values from XML DOM ###
        if self.refresh:
            self.name = member.get("name")
            self.glade_xml.get_widget('member_name_entry').set_text(self.name)
            self.glade_xml.get_widget('member_name_entry').grab_focus()

            self.watchdog = member.get("watchdog")
            if self.watchdog == "yes":
                clu_xml_value = TRUE
            else:
                clu_xml_value = FALSE
            self.glade_xml.get_widget('member_watchdog_checkbutton').set_active(clu_xml_value)

        rc = self.dialog.run()
        self.glade_xml.get_widget('memberDialog_okbutton').disconnect(self.ok)
        self.glade_xml.get_widget('memberDialog_cancelbutton').disconnect(self.cancel)
        return rc

    def hide(self):
        return self.dialog.hide()

class memberItem:
    def __init__(self, parent, member):
        ### parent is "membersTreeview" instance ###
        self.parent    = parent
        self.glade_xml = parent.glade_xml
        self.tree_view = parent.tree_view
        self.model     = parent.tree_view.get_model()
        self.columns   = parent.columns
        self.member    = member
        self.dialog    = parent.dialog

    def GUI_load_row_columns(self, member_item, iter):
        row_display_string = member_item.member.get("name") + "  <member>"
        self.model.set(iter,
                       self.columns['COLUMN_KEY'], row_display_string,
                       self.columns['COLUMN_DATA'], member_item)

    def GUI_tree_load_member(self):
        iter = self.model.append(None)
        self.GUI_load_row_columns(self, iter)

        for p in self.member.get_powercontrollers():
            powercontroller_item = powercontrollerDialog.powercontrollerItem(self, p)
            powercontroller_item.GUI_tree_load_powercontroller(iter)

    def get_type(self):
        return "member"

    def get_value_of_key_field(self):
        return self.get("name")

    def get(self, option):
        return self.member.get(option)

    def __modify(self, iter, action="modify"):
        if action == "modify":
            member = self.member
            member_item = self
        else:
            ### Must be an "add" ###
            members_entity = self.parent.members_entity
            member = members_entity.add_member()
            member_item = memberItem(self.parent, member)

        while 1:
            rc = self.dialog.run(member, action)
            if rc == gtk.RESPONSE_OK:
                if action == "add":
                    ### Since when we're adding a member we're at the top
                    ### of the tree, give append "None" to signify we're
                    ### appending to the root.
                    iter = self.model.append(None)

                self.GUI_load_row_columns(member_item, iter)

            elif rc == gtk.RESPONSE_CANCEL:
                if action == "add":
                    members_entity.delete_member_by_id(member.get("id"))
            else:
                ### rc was gtk.RESPONSE_REJECT:
                ###   1. User got a FATAL error and clicked 'OK' (only option)
                ###      so display same dialog again,
                ###        OR
                ###   2. User got a WARNING and clicked 'Cancel' to go back
                ###      and re-enter.
                continue
            break

        self.dialog.hide()

    def add(self, iter):
        self.__modify(iter, "add")

    def add_child(self, iter):
        powercontroller = self.member.add_powercontroller()
        powercontroller_item = powercontrollerDialog.powercontrollerItem(self, powercontroller)
        powercontroller_dialog = powercontrollerDialog.powercontrollerDialog(self.glade_xml, powercontroller)

        action = "add"
        while 1:
            rc = powercontroller_dialog.run(powercontroller, action)
            if rc == gtk.RESPONSE_OK:
                iter = self.model.append(iter)

                powercontroller_item.GUI_load_row_columns(powercontroller_item, iter)
                break
            if rc == gtk.RESPONSE_CANCEL:
                self.member.delete_powercontroller_by_id(powercontroller.get("id"))
                break

        powercontroller_dialog.hide()

    def properties(self, iter):
        self.__modify(iter)

    def delete(self):
        members_entity = self.parent.members_entity
        try:
            members_entity.delete_member(self.member.get("name"))
        except OptionError, e:
            errorMessage(e.getMessage())
            return FALSE # delete failed
        return TRUE      # delete succeeded
