#!/usr/bin/python2.2

## nfsData.py - Contains the nfs data structure needed for redhat-config-filesharing
## Copyright (C) 2002 Red Hat, Inc.
## Copyright (C) 2002 Brent Fox <bfox@redhat.com>
## Copyright (C) 2002 Dan Walsh <dwalsh@redhat.com>

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
from sharedfilesystem import *

def stripDir(dir):
    d=dir.strip()
    if d!="" and d[-1]=='/':
        return d[:-1]
    else:
        return d
        
class hostsData:
    def __init__(self,hosts,options=""):
        self.setHosts(hosts)
        self.readonly = TRUE
        self.insecure = FALSE
        self.async = FALSE
        self.no_wdelay = FALSE
        self.no_subtree_check = FALSE
        self.insecure_locks = FALSE
        self.no_root_squash = FALSE
        self.all_squash = FALSE
        self.anonuid = ""
        self.anongid = ""
        self.options=options
        optionTokens = self.options.split(",")
        for token in optionTokens:                    
            if token == "ro":
                self.setReadOnly(TRUE)
            elif token == "rw":
                self.setReadOnly(FALSE)
            elif token == "insecure":
                self.set1024(TRUE)
            elif token == "sync":
                self.setAsync(FALSE)
            elif token == "async":
                self.setAsync(TRUE)
            elif token == "no_wdelay":
                self.setForceSync(TRUE)
            elif token == "no_subtree_check":
                self.setSubtree(TRUE)
            elif token == "insecure_locks" or token == "no_auth_nlm":
                self.setInsecureLocks(TRUE)
            elif token == "no_root_squash":
                self.setLocalRoot(TRUE)
            elif token == "all_squash":
                self.setAnonymous(TRUE)
            elif token[:7] == "anonuid":
                str, uid = token.split("=")
                self.setAnonUID(uid)
            elif token[:7] == "anongid":
                str, gid = token.split("=")
                self.setAnonGID(gid)

    def getOptions(self):
        return self.options
    
    def setHosts(self, data):
        if data=="":
            self.hosts = "*"
        else:
            self.hosts = data

    def getHosts(self):
        return self.hosts

    def setReadOnly(self, data):
        self.readonly = data

    def set1024(self, data):
        self.insecure = data

    def setAsync(self, data):
        self.async = data                

    def setForceSync(self, data):
        self.no_wdelay = data

    def setSubtree(self, data):
        self.no_subtree_check = data

    def setInsecureLocks(self, data):
        self.insecure_locks = data

    def setLocalRoot(self, data):
        self.no_root_squash = data

    def setAnonymous(self, data):
        self.all_squash = data

    def setAnonUID(self, data):
        self.anonuid = data

    def setAnonGID(self, data):
        self.anongid = data

    def isReadOnly(self):
        return self.readonly

    def allows1024(self):
        return self.insecure

    def isAsync(self):
        return self.async                

    def isForceSync(self):
        return self.no_wdelay

    def isSubtree(self):
        return self.no_subtree_check

    def isInsecureLocks(self):
        return self.insecure_locks

    def allowsLocalRoot(self):
        return self.no_root_squash

    def isAnonymous(self):
        return self.all_squash

    def getAnonUID(self):
        return self.anonuid

    def getAnonGID(self):
        return self.anongid

    def get_str(self):
        str=self.getHosts() + "("
        if self.isAsync():
            str=str+"async"
        else:
            str=str+"sync"

        if self.isReadOnly():
            str="%s,%s" % (str,"ro")
        else:
            str="%s,%s" % (str,"rw")

        if self.allows1024():
            str="%s,%s" % (str,"insecure")

        if self.isForceSync():
            str="%s,%s" % (str,"no_wdelay")

        if self.isSubtree():
            str="%s,%s" % (str,"no_subtree_check")

        if self.isInsecureLocks():
            str="%s,%s" % (str,"insecure_locks")

        if self.allowsLocalRoot():
            str="%s,%s" % (str,"no_root_squash")

        if self.isAnonymous():
            str="%s,%s" % (str,"all_squash")

        tmp=self.getAnonUID()
        if tmp!=None and tmp!="":
            str="%s,anonuid=%s" % (str,tmp)

        tmp=self.getAnonGID()
        if tmp!=None and tmp!="":
            str="%s,anongid=%s" % (str,tmp)
        str=str+ ")"

        return str
    
class nfsData:
    def __init__(self,line):
        self.line = line
        self.comment=0
        self.directory = None
        self.hosts=[]
        line = line.strip()
        if line=="" or line[0] == '#':
            self.comment=1
        else:
            tokens = line.split()
            self.setDirectory(tokens[0])

            for token in tokens[1:]:
                l=token.split("(")
                hostname=l[0]
                if len(l) > 1:
                    options = l[1].replace(")", "")
                else:
                    options=""
                self.hosts.append(hostsData(hostname,options))

    def setDirectory(self, data):
        self.directory = stripDir(data)

    def isExported(self):
        return self.directory != None

    def getDirectory(self):
        return self.directory

    def add_host(self,host):
        self.hosts.append(host)

    def get_host_str(self):
        if len(self.hosts)>0:
            host_str=self.hosts[0].getHosts()
            for i in self.hosts[1:]:
                host_str="%s,%s" % (host_str,i.getHosts())
        else:
            host_str="*"
        return host_str

    def get_hosts(self):
        return self.hosts

    def set_hosts(self,hosts):
        self.hosts =hosts

    def get_line(self):
        if self.comment:
            return self.line
        else:
            str=self.getDirectory() + "\t "
            for i in self.hosts:
                str = str + i.get_str() + " "
            return str

    def getOptions(self):
        if not self.comment:
            str=""
            for i in self.hosts:
                str = str + i.get_str() + " "
            return str
