#!/usr/bin/python

## nfsexportDialog.py

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import gtk
import gtk.glade
import gettext

from clusterpkg.cluster_functions     import OptionError
from clusterpkg.cluster_functions_gui import errorMessage

import clientDialog

##
## I18N
##

class nfsexportDialog: #(gtk.Dialog):
    def __init__(self, glade_xml, nfsexport):
        self.glade_xml = glade_xml
        self.nfsexport = nfsexport

        self.dialog = glade_xml.get_widget('nfsexportDialog')

        self.refresh = gtk.TRUE

    def on_cancelbutton_clicked(self, button):
        self.refresh = gtk.TRUE

    def on_okbutton_clicked(self, button):

        ### "get" the new values from the Gui widgets ###

        name = self.glade_xml.get_widget('nfsexport_directoryname_entry').get_text()

        ### If user didn't update any fields, then just return.
        if name == self.name:
            self.dialog.response(gtk.RESPONSE_OK)
            return

        options = { "name" : name }

        self.refresh = gtk.TRUE
        rc = gtk.RESPONSE_OK
        try:
            ### Invoke nfsexport(ClusterEntity) "check" function to validate
            ### option.
            self.nfsexport.validate(options)
        except OptionError, e:
            if e.getSeverity() == 'FATAL':
                errorMessage(e.getMessage())
                self.refresh = gtk.FALSE # Don't refresh - leave bad values
                                         # on screen.
                rc = gtk.RESPONSE_REJECT

            elif e.getSeverity() == 'WARNING':
                msg = _("%s\n\nPress 'OK' to Continue, or 'Cancel' to Exit.") % e.getMessage()
                rc = warningMessage(msg)
                if rc != gtk.RESPONSE_OK:
                   self.refresh = gtk.FALSE # Don't refresh - leave bad values
                                            # on screen.
                   rc = gtk.RESPONSE_REJECT # Need to differentiate this
                                            # 'Cancel' from the main Dialog
                                            # 'Cancel', so make this rc
                                            # gtk.RESPONSE_REJECT.

        if rc == gtk.RESPONSE_OK:
            ### Validation passed - now 'set' DOM XML options ###
            self.nfsexport.set("name", name)
            self.nfsexport.change_action_to_modify()

        self.dialog.response(rc)
        return

    def run(self, nfsexport, action):
        self.nfsexport = nfsexport
        self.action    = action

        ### Now connect the buttons to the functions ###
        self.ok     = self.glade_xml.get_widget('nfsexportDialog_okbutton').connect("clicked", self.on_okbutton_clicked)
        self.cancel = self.glade_xml.get_widget('nfsexportDialog_cancelbutton').connect("clicked", self.on_cancelbutton_clicked)

        ### Load screen widgets with data from XML DOM for selected device ###

        if self.refresh:
            self.name = nfsexport.get("name")
            self.glade_xml.get_widget('nfsexport_directoryname_entry').set_text(self.name)

        rc = self.dialog.run()
        self.glade_xml.get_widget('nfsexportDialog_okbutton').disconnect(self.ok)
        self.glade_xml.get_widget('nfsexportDialog_cancelbutton').disconnect(self.cancel)
        return rc

    def hide(self):
        return self.dialog.hide()

class nfsexportItem:
    def __init__(self, parent, nfsexport):
        self.parent    = parent
        self.glade_xml = parent.glade_xml
        self.treeview  = parent.treeview
        self.model     = parent.treeview.get_model()
        self.columns   = parent.columns
        self.nfsexport = nfsexport

        self.dialog = nfsexportDialog(self.glade_xml, nfsexport)

    def GUI_load_row_columns(self, nfsexport_item, iter):
        row_display_string = _("%s  <nfsexport>") % nfsexport_item.nfsexport.get("name")
        self.model.set(iter,
                       self.columns['COLUMN_KEY'], row_display_string,
                       self.columns['COLUMN_DATA'], nfsexport_item)

    def GUI_tree_load_nfsexport(self, iter):
        nfsexport_iter = self.model.append(iter)
        self.GUI_load_row_columns(self, nfsexport_iter)

        for c in self.nfsexport.get_clients():
            client_item = clientDialog.clientItem(self, c)
            client_item.GUI_tree_load_client(nfsexport_iter)

    def get_type(self):
        return "nfsexport"

    def get_value_of_key_field(self):
        return self.get("name") # NFS Export key field is "name"

    def get(self, option):
        return self.nfsexport.get(option)

    def __modify(self, iter, action="modify"):
        if action == "modify":
            nfsexport = self.nfsexport
            nfsexport_item = self
        else:
            ### Must be an "add" ###
            device_item = self.parent
            device_entity = device_item.device

            nfsexport = device_entity.add_nfsexport()
            nfsexport_item = nfsexportItem(self.parent, nfsexport)

        while 1:
            rc = self.dialog.run(nfsexport, action)
            if rc == gtk.RESPONSE_OK:
                if action == "add":
                    ### model.append needs the "iter" of the *parent* of the
                    ### thing that is being appended.
                    parent_iter = iter
                    iter = self.model.append(parent_iter)

                self.GUI_load_row_columns(nfsexport_item, iter)

            elif rc == gtk.RESPONSE_CANCEL:
                if action == "add":
                    device_entity.delete_nfsexport_by_id(nfsexport.get("id"))
            else:
                ### rc was gtk.RESPONSE_REJECT:
                ###   1. User got a FATAL error and clicked 'OK' (only option)
                ###      so display same dialog again,
                ###        OR
                ###   2. User got a WARNING and clicked 'Cancel' to go back
                ###      and re-enter.
                continue
            break

        self.dialog.hide()

    def add(self, iter):
        self.__modify(iter, "add")

    def add_host(self, iter,host):
        client = self.nfsexport.add_client()
        client_item = clientDialog.clientItem(self, client)
        iter = self.model.append(iter)
        client.set("name",host.getHosts())
        client.set("options",host.getOptions())
        client_item.GUI_load_row_columns(client_item, iter)
        
    def add_child(self, iter):
        client = self.nfsexport.add_client()
        client_item = clientDialog.clientItem(self, client)
        client_dialog = clientDialog.clientDialog(self.glade_xml, client)
        action = "add"
        while 1:
            rc = client_dialog.run(client, action)
            if rc == gtk.RESPONSE_OK:
                iter = self.model.append(iter)
                client_item.GUI_load_row_columns(client_item, iter)
                break
            if rc == gtk.RESPONSE_CANCEL:
                self.nfsexport.delete_client_by_id(client.get("id"))
                break
        client_dialog.hide()

    def properties(self, iter):
        self.__modify(iter)

    def delete(self):
        device_item = self.parent
        device_entity = device_item.device
        device_entity.delete_nfsexport_by_id(self.nfsexport.get("id"))
