import sys

import nfsBackend
import nfsData
import gtk
import gtk.glade
import signal
import os
import string
import gettext
import gobject
import copy

import serviceDialog
from clusterpkg.service_module    import service
from clusterpkg.cluster_functions import cluStat
from clusterpkg.util_smbconf_module import getAbsShareFilename

##
## I18N
## 

_=gettext.gettext

COLUMN_KEY = 0 # 0 for 1st column.  This column(the only column displayed)
               # will contain the key field(that uniquely identifies that
               # object) for the object in that row.
COLUMN_DATA = 1

TRUE = 1
FALSE = 0

class servicesTreeview:
    def __init__(self, glade_xml, services_entity):
        self.services_entity = services_entity
        self.glade_xml = glade_xml
        self.treeview = self.glade_xml.get_widget ('servicesTreeview')
        self.model = gtk.TreeStore(gobject.TYPE_STRING,
                                   gobject.TYPE_PYOBJECT)
        self.treeview.set_model(self.model)

        column = gtk.TreeViewColumn(_("Name"),
                                    gtk.CellRendererText(),
                                    text=COLUMN_KEY)
        self.treeview.append_column(column)

        self.treeview.get_selection().connect("changed",
                                            self.on_treeview_selection_changed)
        self.treeview.connect("row_activated",
                               self.properties, None)

        self.columns = { "COLUMN_KEY": COLUMN_KEY,
                         "COLUMN_DATA" : COLUMN_DATA }

        self.treeview.get_selection().select_path ((0,))
        self.dialog = serviceDialog.serviceDialog(self.glade_xml)
        self.populate_treeview()

    def populate_treeview(self):
        self.model.clear()
        for s in self.services_entity.get_list():
            service_item = serviceDialog.serviceItem(self, s)
            service_item.GUI_tree_load_service()

    def on_treeview_selection_changed(self, selection):
        result = selection.get_selected ()
        if result == None:
            return 
        (model, iter) = result
        if iter == None:
            return
        item = model.get_value(iter, COLUMN_DATA)
        if item.get_type() == "client" or \
           item.get_type() == "service_ipaddress":
            self.glade_xml.get_widget('add_child_button').set_sensitive(gtk.FALSE)
            self.glade_xml.get_widget('add_child_menu').set_sensitive(gtk.FALSE)
        else:
            self.glade_xml.get_widget('add_child_button').set_sensitive(gtk.TRUE)
            self.glade_xml.get_widget('add_child_menu').set_sensitive(gtk.TRUE)

        if item.get_type() == "device":
            self.glade_xml.get_widget('bulkload_menu').set_sensitive(gtk.TRUE)
        else:
            self.glade_xml.get_widget('bulkload_menu').set_sensitive(gtk.FALSE)

    def add(self):
        iter = None
        result = self.treeview.get_selection().get_selected ()
        if result != None:
            (model, iter) = result
        if iter != None:
            ### Want to add an item at the same level as the currently
            ### selected item.  So call item.add with the iter of the
            ### current iter's parent, since that parent will also
            ### be the parent of the item being added.
            item = model.get_value(iter, COLUMN_DATA)
            newitem = item.add(model.iter_parent(iter))
        else:
            item = serviceDialog.serviceItem(self, None)
            newitem = item.add(None)

        if newitem != None:
             model.set(iter,
                       COLUMN_KEY, newitem.get("name"),
                       COLUMN_DATA, newitem)

    def add_child(self):
        #print "servicesTreeview add_child TOP"
        iter=None
        result = self.treeview.get_selection().get_selected ()
        if result != None:
            (model, iter) = result
        if iter != None:
            item = model.get_value(iter, COLUMN_DATA)
        else:
            item = serviceDialog.serviceItem(self, None)

        ### Since the current item will be the parent of the item being
        ### added, pass the current item's iter to item.add_child.
        newitem = item.add_child(iter)
        if newitem != None:
            model.set(iter,
                      COLUMN_KEY, newitem.get("name"),
                      COLUMN_DATA, newitem)

    def serviceForItem(self, item):
        type = item.get_type()
        service = None
        if type == "service":
            service = item.service
        elif type == "service_ipaddress":
            type = "service ip address"
            service_item = item.parent
            service = service_item.service
        elif type == "device":
            service_item = item.parent
            service = service_item.service
        elif type == "nfsexport":
            device_item = item.parent
            service_item = device_item.parent
            service = service_item.service
        elif type == "client":
            nfsexport_item = item.parent
            device_item = nfsexport_item.parent
            service_item = device_item.parent
            service = service_item.service
        return (type, service)

    def delete(self):
        result = self.treeview.get_selection().get_selected ()
        if result == None:
            return 
        (model, iter) = result
        if iter == None:
            return
        item = model.get_value(iter, COLUMN_DATA)
        item_type, service = self.serviceForItem(item)
        service_id = service.get("id")
        clustat = cluStat()
        if clustat.serviceIsRunning(service_id):
            message =_("Service '%s' must be disabled before '%s' delete allowed") % (service.get("name"), item_type)
            dlg = gtk.MessageDialog(
                parent         = None,
                flags          = gtk.DIALOG_MODAL,
                type           = gtk.MESSAGE_ERROR,
                buttons        = gtk.BUTTONS_OK,
                message_format = message)
            dlg.set_position(gtk.WIN_POS_MOUSE)
            dlg.show_all()
            rc = dlg.run()
            dlg.destroy()
            return
        message=_("Are you sure you want to remove %s '%s'?") % (item.get_type(), item.get_value_of_key_field())
        dlg = gtk.MessageDialog(None, 0, gtk.MESSAGE_INFO,
                                gtk.BUTTONS_YES_NO,
                                message)
        dlg.set_position(gtk.WIN_POS_MOUSE)
        dlg.show_all()
        rc = dlg.run()
        dlg.destroy()
        if rc==gtk.RESPONSE_YES:
            ### remove "iter" from Gui ###
            model.remove(iter)

            ### remove "item" from DOM XML ###
            item.delete()

    def properties(self, *args):
        result = self.treeview.get_selection().get_selected ()
        if result == None:
            return 
        (model, iter) = result
        if iter == None:
            return
        item = model.get_value (iter, COLUMN_DATA)
        item.properties(iter)

    def on_bulkloadNFS(self):
        result = self.treeview.get_selection().get_selected ()
        if result == None:
            return
        (model, iter) = result
        if iter == None:
            return
        item = model.get_value(iter, COLUMN_DATA)
        dlg=self.glade_xml.get_widget('bulkLoadNFSDialog')
        dlg.show_all()
        dlg.set_filename("/etc/exports")
        rc = dlg.run()
        if rc == gtk.RESPONSE_OK:
            nfs=nfsBackend.nfsBackend(dlg.get_filename())
            item.clear_failed_exports()
            for n in nfs.getEntries():
                if n.isExported():
                    newitem = item.add_export(iter,n.getDirectory(),n.get_hosts())
            failed_exports = item.get_failed_exports()
            if len(failed_exports) > 0:
                failure_dlg = self.glade_xml.get_widget('failedNFSBulkLoadDialog')
                text_widget = self.glade_xml.get_widget('nfs_bulkload_failures_textview')
                text = string.join(failed_exports, "\n")
                text_widget.get_buffer().set_text(text)
                failure_dlg.show_all()
                rc = failure_dlg.run()
                failure_dlg.hide()
        dlg.hide()
        print "\nservicesTreeview on_bulkloadNFS END!"
        print "===================================="
        print self.services_entity.node.toxml()

    ##########################################################
    # function: shareExists
    # desc:     For the sharename passed in, see if that
    #           sharename exists on any 'device' in the
    #           gui servicesTreeview tree.
    # accepts:  sharename
    # returns:  TRUE - if sharename found on any 'device' in
    #                  existing servicesTreeview gui tree.
    #           FALSE - if sharename not found.
    ##########################################################
    def shareExists(self, sharename):
        def foreach_func (model, path, iter, text):
            node_entity = model.get_value (iter, COLUMN_DATA)
            if node_entity.get_type() == "device":
                if node_entity.device.get("sharename") == sharename:
                    return TRUE
        self.treeview.get_model().foreach (foreach_func, None)
        return FALSE

    #def saveAllShares(self):
    #    #print "servicesTreeview saveAllShares TOP!"
    #    def foreach_func (model, path, iter, text):
    #        node_entity = model.get_value (iter, COLUMN_DATA)
    #        if node_entity.get_type() == "device":
    #            device = node_entity.device
    #            #print "  device=[%s], device name=[%s], sharename=[%s]" % (device, device.get("name"), device.get("sharename"))
    #            if device.get("sharename") == "":
    #                return
    #
    #            (sharename, ipaddresses, mountpoint, action) = \
    #                device.smbconf.writeSambaShareFile()
    #            #print "smb sharename  =[%s]" % sharename
    #            #print "smb ipaddresses=[%s]" % ipaddresses
    #            #print "smb mountpoint =[%s]" % mountpoint
    def saveAllShares(self, device_share_dict_before):
        #print "\nservicesTreeview saveAllShares TOP:"
        def foreach_func (model, path, iter, text):
            node_entity = model.get_value (iter, COLUMN_DATA)
            if node_entity.get_type() != "device":
                return ### skip this entity - it's not a "device"

            ### Working on a 'Device' ###
            device = node_entity.device
            gui_device_name      = device.get("name")
            gui_device_sharename = device.get("sharename")
            if device_share_dict_before.has_key(gui_device_name):
                prev_sharename = device_share_dict_before[gui_device_name]
                ### Device still exists ###
                if gui_device_sharename != "":
                    ### Samba Share Name was specified - write file ###
                    (sharename, ipaddresses, mountpoint, action) = \
                        device.smbconf.writeSambaShareFile()
                    ### if sharename changed from non-null to non-null,
                    ### remove old file ###
                    if prev_sharename != gui_device_sharename and \
                       prev_sharename != "":
                        try:
                            os.remove(getAbsShareFilename(prev_sharename))
                        except OSError, e:
                            pass
                else:
                    ### screen sharename is null ###
                    if prev_sharename != "":
                        ### user removed the sharename on this device ###
                        device.smbconf.init_file_dict()
                        
                del devices_left[gui_device_name]
            else:
                ### New Device - if sharename was filled in, write file ###
                if gui_device_sharename != "":
                    (sharename, ipaddresses, mountpoint, action) = \
                        device.smbconf.writeSambaShareFile()

        devices_left = copy.deepcopy(device_share_dict_before)

        self.treeview.get_model().foreach (foreach_func, None)

        ### Devices left on 'devices_left' were present before, but not
        ### after, which means that each of these devices was deleted
        ### by the user.
        #for device_name, share_name in devices_left.items():
        #    ### deleted device ###
        #    print "   device=[%s] sharename=[%s] was deleted" % (device_name, share_name)
